from unittest import TestCase

from pkg_resources import resource_filename

import pyproteininference

TEST_DATABASE = resource_filename("pyproteininference", "../tests/data/test_database.fasta")
PARAMETER_FILE = resource_filename("pyproteininference", "../tests/data/test_params_inclusion.yaml")


class TestDigest(TestCase):
    def test_pyteomics_digest(self):

        # STEP 1: Load parameter file #
        # STEP 1: Load parameter file #
        # STEP 1: Load parameter file #
        protein_inference_parameters = pyproteininference.parameters.ProteinInferenceParameter(
            yaml_param_filepath=PARAMETER_FILE
        )

        # STEP 2: Start with running an In Silico Digestion #
        # STEP 2: Start with running an In Silico Digestion #
        # STEP 2: Start with running an In Silico Digestion #
        digest = pyproteininference.in_silico_digest.PyteomicsDigest(
            database_path=TEST_DATABASE,
            digest_type=protein_inference_parameters.digest_type,
            missed_cleavages=protein_inference_parameters.missed_cleavages,
            reviewed_identifier_symbol=protein_inference_parameters.reviewed_identifier_symbol,
            max_peptide_length=None,
            id_splitting=True,
        )
        digest.digest_fasta_database()

        self.assertSetEqual(
            digest.swiss_prot_protein_set,
            {
                "ARAF_HUMAN|P10398",
                "##ARAF_HUMAN|##P10398",
                "##BRAF_HUMAN|##P15056",
                "HNRPU_HUMAN|Q00839",
                "BRAF_HUMAN|P15056",
                "##RPOC_SHIF8|##Q0SY12",
                "TCAF1_HUMAN|Q9Y4C2-2",
                "##TCAF2_HUMAN|##A6NFQ2-3",
                "RAF1_HUMAN|P04049",
                "RPOC_SHIF8|Q0SY12",
                "##TCAF2_HUMAN|##A6NFQ2",
                "TCAF2_HUMAN|A6NFQ2-3",
                "##RAF1_HUMAN|##P04049",
                "##TCAF1_HUMAN|##Q9Y4C2-2",
                "TCAF2_HUMAN|A6NFQ2",
                "##TCAF1_HUMAN|##Q9Y4C2",
                "##HNRPU_HUMAN|##Q00839",
                "##TCAF2_HUMAN|##A6NFQ2-2",
                "TCAF2_HUMAN|A6NFQ2-2",
                "TCAF1_HUMAN|Q9Y4C2",
            },
        )

        self.assertSetEqual(
            digest.protein_to_peptide_dictionary["ARAF_HUMAN|P10398"],
            {
                "NIIHRDLKSNNIFLHEGLTVK",
                "FQR",
                "RIGTGSFGTVFRGR",
                "EERPLFPQILATIELLQR",
                "KTR",
                "LLSDCLKFQR",
                "RLLSDCLK",
                "IERSASEPSLHRTQADELPACLLSAAR",
                "AVGTVKVYLPNKQRTVVTVR",
                "VPTVCVDMSTNRQQFYHSVQDLSGGSRQHEAPSNRPLNELLTPQGPSPRTQHCDPEHFPFPAPANAPLQR",
                "FHQHCSSKVPTVCVDMSTNR",
                "GPPANGAEPSR",
                "TQHCDPEHFPFPAPANAPLQR",
                "NEMQVLR",
                "TRHVNILLFMGFMTRPGFAIITQWCEGSSLYHHLHVADTRFDMVQLIDVAR",
                "WSGAQPLEQPSGSVLWMAAEVIRMQDPNPYSFQSDVYAYGVVLYELMTGSLPYSHIGCR",
                "DGMSVYDSLDKALK",
                "TRWSGAQPLEQPSGSVLWMAAEVIR",
                "LLSDCLK",
                "VLKVSQPTAEQAQAFKNEMQVLR",
                "MQDPNPYSFQSDVYAYGVVLYELMTGSLPYSHIGCRDQIIFMVGRGYLSPDLSKISSNCPK",
                "WSGAQPLEQPSGSVLWMAAEVIRMQDPNPYSFQSDVYAYGVVLYELMTGSLPYSHIGCRDQIIFMVGR",
                "AMR",
                "GSPSPASVSSGR",
                "WHGDVAVKVLKVSQPTAEQAQAFKNEMQVLR",
                "TR",
                "SPHSKSPAEQR",
                "SLADDKK",
                "NLGYRDSGYYWEVPPSEVQLLKRIGTGSFGTVFR",
                "WHGDVAVKVLK",
                "NIIHRDLKSNNIFLHEGLTVKIGDFGLATVK",
                "TRWSGAQPLEQPSGSVLWMAAEVIRMQDPNPYSFQSDVYAYGVVLYELMTGSLPYSHIGCRDQIIFMVGR",
                "EERPLFPQILATIELLQRSLPKIER",
                "EERPLFPQILATIELLQRSLPKIERSASEPSLHR",
                "MQDPNPYSFQSDVYAYGVVLYELMTGSLPYSHIGCRDQIIFMVGR",
                "IGDFGLATVKTRWSGAQPLEQPSGSVLWMAAEVIRMQDPNPYSFQSDVYAYGVVLYELMTGSLPYSHIGCR",
                "FQREERPLFPQILATIELLQRSLPKIER",
                "TVTAWDTAIAPLDGEELIVEVLEDVPLTMHNFVRKTFFSLAFCDFCLK",
                "KSPHSKSPAEQR",
                "QTAQGMDYLHAKNIIHR",
                "SNNIFLHEGLTVKIGDFGLATVK",
                "ERK",
                "TVVTVRDGMSVYDSLDK",
                "VRGLNQDCCVVYRLIK",
                "AVGTVK",
                "QQFYHSVQDLSGGSRQHEAPSNRPLNELLTPQGPSPRTQHCDPEHFPFPAPANAPLQR",
                "KSPHSKSPAEQRER",
                "LLSDCLKFQREERPLFPQILATIELLQR",
                "WHGDVAVKVLKVSQPTAEQAQAFK",
                "TVTAWDTAIAPLDGEELIVEVLEDVPLTMHNFVRKTFFSLAFCDFCLKFLFHGFR",
                "RIGTGSFGTVFR",
                "R",
                "KSLADDK",
                "ISSNCPKAMRRLLSDCLK",
                "MEPPR",
                "NEMQVLRKTRHVNILLFMGFMTRPGFAIITQWCEGSSLYHHLHVADTR",
                "CQTCGYKFHQHCSSK",
                "QRTVVTVRDGMSVYDSLDK",
                "SASEPSLHR",
                "NLGYR",
                "ER",
                "KTFFSLAFCDFCLKFLFHGFR",
                "QHEAPSNRPLNELLTPQGPSPRTQHCDPEHFPFPAPANAPLQR",
                "ISSNCPKAMRR",
                "TFFSLAFCDFCLKFLFHGFR",
                "STSTPNVHMVSTTAPMDSNLIQLTGQSFSTDAAGSR",
                "DQIIFMVGR",
                "KSPHSK",
                "DLKSNNIFLHEGLTVKIGDFGLATVK",
                "NIIHRDLK",
                "GRWHGDVAVK",
                "IGTGSFGTVFRGRWHGDVAVK",
                "QTAQGMDYLHAK",
                "VSQPTAEQAQAFKNEMQVLR",
                "DGMSVYDSLDK",
                "EPPRGPPANGAEPSR",
                "EPPRGPPANGAEPSRAVGTVKVYLPNK",
                "ALKVRGLNQDCCVVYRLIK",
                "STSTPNVHMVSTTAPMDSNLIQLTGQSFSTDAAGSRGGSDGTPRGSPSPASVSSGR",
                "IRSTSTPNVHMVSTTAPMDSNLIQLTGQSFSTDAAGSRGGSDGTPRGSPSPASVSSGR",
                "SPAEQRERKSLADDK",
                "FLFHGFRCQTCGYK",
                "QQFYHSVQDLSGGSRQHEAPSNRPLNELLTPQGPSPR",
                "FDMVQLIDVAR",
                "NLGYRDSGYYWEVPPSEVQLLK",
                "QTAQGMDYLHAKNIIHRDLKSNNIFLHEGLTVK",
                "MEPPRGPPANGAEPSR",
                "LLSDCLKFQREERPLFPQILATIELLQRSLPK",
                "GLNQDCCVVYRLIKGR",
                "GGSDGTPRGSPSPASVSSGR",
                "SPAEQR",
                "GPPANGAEPSRAVGTVKVYLPNKQR",
                "TVTAWDTAIAPLDGEELIVEVLEDVPLTMHNFVR",
                "GPPANGAEPSRAVGTVK",
                "WSGAQPLEQPSGSVLWMAAEVIRMQDPNPYSFQSDVYAYGVVLYELMTGSLPYSHIGCRDQIIFMVGRGYLSPDLSK",
                "CQTCGYK",
                "KSLADDKK",
                "RLLSDCLKFQR",
                "VR",
                "KVK",
                "DSGYYWEVPPSEVQLLKR",
                "TRHVNILLFMGFMTRPGFAIITQWCEGSSLYHHLHVADTRFDMVQLIDVARQTAQGMDYLHAK",
                "FHQHCSSKVPTVCVDMSTNRQQFYHSVQDLSGGSR",
                "KSLADDKKK",
                "EERPLFPQILATIELLQRSLPK",
                "TVVTVRDGMSVYDSLDKALK",
                "KTFFSLAFCDFCLK",
                "GGSDGTPR",
                "ALKVR",
                "VKNLGYRDSGYYWEVPPSEVQLLK",
                "QHEAPSNRPLNELLTPQGPSPRTQHCDPEHFPFPAPANAPLQRIRSTSTPNVHMVSTTAPMDSNLIQLTGQSFSTDAAGSR",
                "IRSTSTPNVHMVSTTAPMDSNLIQLTGQSFSTDAAGSR",
                "FHQHCSSKVPTVCVDMSTNRQQFYHSVQDLSGGSRQHEAPSNRPLNELLTPQGPSPR",
                "VPTVCVDMSTNRQQFYHSVQDLSGGSRQHEAPSNRPLNELLTPQGPSPR",
                "QHEAPSNRPLNELLTPQGPSPRTQHCDPEHFPFPAPANAPLQRIR",
                "DSGYYWEVPPSEVQLLKRIGTGSFGTVFRGR",
                "VSQPTAEQAQAFKNEMQVLRKTR",
                "GRKTVTAWDTAIAPLDGEELIVEVLEDVPLTMHNFVR",
                "KTVTAWDTAIAPLDGEELIVEVLEDVPLTMHNFVRK",
                "TRHVNILLFMGFMTRPGFAIITQWCEGSSLYHHLHVADTR",
                "FHQHCSSK",
                "TQADELPACLLSAARLVP",
                "DGMSVYDSLDKALKVRGLNQDCCVVYR",
                "CQTCGYKFHQHCSSKVPTVCVDMSTNRQQFYHSVQDLSGGSR",
                "KTRHVNILLFMGFMTRPGFAIITQWCEGSSLYHHLHVADTRFDMVQLIDVAR",
                "VYLPNKQRTVVTVR",
                "VK",
                "STSTPNVHMVSTTAPMDSNLIQLTGQSFSTDAAGSRGGSDGTPRGSPSPASVSSGRK",
                "DQIIFMVGRGYLSPDLSKISSNCPK",
                "AVGTVKVYLPNK",
                "ISSNCPKAMR",
                "TQHCDPEHFPFPAPANAPLQRIRSTSTPNVHMVSTTAPMDSNLIQLTGQSFSTDAAGSRGGSDGTPR",
                "RIGTGSFGTVFRGRWHGDVAVK",
                "GLNQDCCVVYRLIK",
                "NEMQVLRKTR",
                "IGTGSFGTVFR",
                "IGDFGLATVKTRWSGAQPLEQPSGSVLWMAAEVIR",
                "TFFSLAFCDFCLK",
                "SLPKIER",
                "IGTGSFGTVFRGRWHGDVAVKVLK",
                "GR",
                "IRSTSTPNVHMVSTTAPMDSNLIQLTGQSFSTDAAGSRGGSDGTPR",
                "KTFFSLAFCDFCLKFLFHGFRCQTCGYK",
                "EPPRGPPANGAEPSRAVGTVK",
                "FLFHGFRCQTCGYKFHQHCSSK",
                "VRGLNQDCCVVYRLIKGR",
                "QQFYHSVQDLSGGSRQHEAPSNRPLNELLTPQGPSPRTQHCDPEHFPFPAPANAPLQRIR",
                "ERKSLADDK",
                "FDMVQLIDVARQTAQGMDYLHAK",
                "TQADELPACLLSAAR",
                "QRTVVTVRDGMSVYDSLDKALK",
                "FLFHGFRCQTCGYKFHQHCSSKVPTVCVDMSTNR",
                "SPHSK",
                "GRKTVTAWDTAIAPLDGEELIVEVLEDVPLTMHNFVRK",
                "TRWSGAQPLEQPSGSVLWMAAEVIRMQDPNPYSFQSDVYAYGVVLYELMTGSLPYSHIGCR",
                "SLPK",
                "DLKSNNIFLHEGLTVK",
                "VYLPNK",
                "GGSDGTPRGSPSPASVSSGRK",
                "WSGAQPLEQPSGSVLWMAAEVIR",
                "SASEPSLHRTQADELPACLLSAAR",
                "NLGYRDSGYYWEVPPSEVQLLKR",
                "SNNIFLHEGLTVK",
                "GYLSPDLSKISSNCPK",
                "GLNQDCCVVYR",
                "VLKVSQPTAEQAQAFKNEMQVLRK",
                "SLADDK",
                "MQDPNPYSFQSDVYAYGVVLYELMTGSLPYSHIGCR",
                "KKVK",
                "ALK",
                "LIKGRK",
                "GSPSPASVSSGRKSPHSK",
                "K",
                "DQIIFMVGRGYLSPDLSK",
                "TQHCDPEHFPFPAPANAPLQRIRSTSTPNVHMVSTTAPMDSNLIQLTGQSFSTDAAGSR",
                "GYLSPDLSKISSNCPKAMRR",
                "FQREERPLFPQILATIELLQRSLPK",
                "DGMSVYDSLDKALKVR",
                "AVGTVKVYLPNKQR",
                "GPPANGAEPSRAVGTVKVYLPNK",
                "TVTAWDTAIAPLDGEELIVEVLEDVPLTMHNFVRK",
                "QR",
                "DSGYYWEVPPSEVQLLKRIGTGSFGTVFR",
                "GGSDGTPRGSPSPASVSSGRKSPHSK",
                "KVKNLGYR",
                "NEMQVLRK",
                "AMRRLLSDCLK",
                "TFFSLAFCDFCLKFLFHGFRCQTCGYK",
                "LIKGRKTVTAWDTAIAPLDGEELIVEVLEDVPLTMHNFVR",
                "KKVKNLGYR",
                "HVNILLFMGFMTRPGFAIITQWCEGSSLYHHLHVADTR",
                "CQTCGYKFHQHCSSKVPTVCVDMSTNR",
                "GYLSPDLSKISSNCPKAMR",
                "KVKNLGYRDSGYYWEVPPSEVQLLK",
                "HVNILLFMGFMTRPGFAIITQWCEGSSLYHHLHVADTRFDMVQLIDVARQTAQGMDYLHAK",
                "GRWHGDVAVKVLKVSQPTAEQAQAFK",
                "SPAEQRER",
                "RLLSDCLKFQREERPLFPQILATIELLQR",
                "GRK",
                "SNNIFLHEGLTVKIGDFGLATVKTR",
                "VRGLNQDCCVVYR",
                "HVNILLFMGFMTRPGFAIITQWCEGSSLYHHLHVADTRFDMVQLIDVAR",
                "TVVTVR",
                "LIK",
                "TVVTVRDGMSVYDSLDKALKVR",
                "ALKVRGLNQDCCVVYR",
                "GSPSPASVSSGRK",
                "QHEAPSNRPLNELLTPQGPSPR",
                "IER",
                "DLKSNNIFLHEGLTVKIGDFGLATVKTR",
                "VLKVSQPTAEQAQAFK",
                "SLADDKKKVK",
                "IR",
                "SASEPSLHRTQADELPACLLSAARLVP",
                "DSGYYWEVPPSEVQLLK",
                "GRWHGDVAVKVLK",
                "VSQPTAEQAQAFK",
                "VKNLGYR",
                "IGDFGLATVKTR",
                "VYLPNKQR",
                "GYLSPDLSK",
                "VPTVCVDMSTNR",
                "FDMVQLIDVARQTAQGMDYLHAKNIIHRDLK",
                "IGDFGLATVK",
                "AMRRLLSDCLKFQR",
                "SLPKIERSASEPSLHR",
                "IGTGSFGTVFRGR",
                "VYLPNKQRTVVTVRDGMSVYDSLDK",
                "MEPPRGPPANGAEPSRAVGTVKVYLPNK",
                "VSQPTAEQAQAFKNEMQVLRK",
                "KK",
                "FDMVQLIDVARQTAQGMDYLHAKNIIHR",
                "VLK",
                "ISSNCPK",
                "IERSASEPSLHRTQADELPACLLSAARLVP",
                "DLK",
                "AMRR",
                "DQIIFMVGRGYLSPDLSKISSNCPKAMR",
                "TQHCDPEHFPFPAPANAPLQRIR",
                "SPHSKSPAEQRERK",
                "WHGDVAVK",
                "LIKGR",
                "QRTVVTVR",
                "SPAEQRERK",
                "FQREERPLFPQILATIELLQR",
                "QQFYHSVQDLSGGSR",
                "QTAQGMDYLHAKNIIHRDLK",
                "KTVTAWDTAIAPLDGEELIVEVLEDVPLTMHNFVR",
                "FLFHGFR",
                "MQDPNPYSFQSDVYAYGVVLYELMTGSLPYSHIGCRDQIIFMVGRGYLSPDLSK",
                "ERKSLADDKK",
                "SNNIFLHEGLTVKIGDFGLATVKTRWSGAQPLEQPSGSVLWMAAEVIR",
                "EPPR",
                "VKNLGYRDSGYYWEVPPSEVQLLKR",
                "MEPPRGPPANGAEPSRAVGTVK",
                "VPTVCVDMSTNRQQFYHSVQDLSGGSR",
                "KTVTAWDTAIAPLDGEELIVEVLEDVPLTMHNFVRKTFFSLAFCDFCLK",
                "HVNILLFMGFMTRPGFAIITQWCEGSSLYHHLHVADTRFDMVQLIDVARQTAQGMDYLHAKNIIHR",
                "TFFSLAFCDFCLKFLFHGFRCQTCGYKFHQHCSSK",
                "LVP",
                "NIIHR",
                "GSPSPASVSSGRKSPHSKSPAEQR",
                "SLPKIERSASEPSLHRTQADELPACLLSAAR",
                "KTRHVNILLFMGFMTRPGFAIITQWCEGSSLYHHLHVADTR",
                "STSTPNVHMVSTTAPMDSNLIQLTGQSFSTDAAGSRGGSDGTPR",
                "SPHSKSPAEQRER",
                "SLADDKKK",
                "IERSASEPSLHR",
                "GLNQDCCVVYRLIKGRK",
            },
        )

        self.assertEqual(len(digest.peptide_to_protein_dictionary), 5079)

        protein_inference_parameters.missed_cleavages = 2
        digest2 = pyproteininference.in_silico_digest.PyteomicsDigest(
            database_path=TEST_DATABASE,
            digest_type=protein_inference_parameters.digest_type,
            missed_cleavages=protein_inference_parameters.missed_cleavages,
            reviewed_identifier_symbol=protein_inference_parameters.reviewed_identifier_symbol,
            max_peptide_length=None,
            id_splitting=True,
        )
        digest2.digest_fasta_database()

        self.assertSetEqual(
            digest2.swiss_prot_protein_set,
            {
                "ARAF_HUMAN|P10398",
                "##ARAF_HUMAN|##P10398",
                "##BRAF_HUMAN|##P15056",
                "HNRPU_HUMAN|Q00839",
                "BRAF_HUMAN|P15056",
                "##RPOC_SHIF8|##Q0SY12",
                "TCAF1_HUMAN|Q9Y4C2-2",
                "##TCAF2_HUMAN|##A6NFQ2-3",
                "RAF1_HUMAN|P04049",
                "RPOC_SHIF8|Q0SY12",
                "##TCAF2_HUMAN|##A6NFQ2",
                "TCAF2_HUMAN|A6NFQ2-3",
                "##RAF1_HUMAN|##P04049",
                "##TCAF1_HUMAN|##Q9Y4C2-2",
                "TCAF2_HUMAN|A6NFQ2",
                "##TCAF1_HUMAN|##Q9Y4C2",
                "##HNRPU_HUMAN|##Q00839",
                "##TCAF2_HUMAN|##A6NFQ2-2",
                "TCAF2_HUMAN|A6NFQ2-2",
                "TCAF1_HUMAN|Q9Y4C2",
            },
        )

        self.assertSetEqual(
            digest2.protein_to_peptide_dictionary["ARAF_HUMAN|P10398"],
            {
                "QTAQGMDYLHAKNIIHR",
                "VLKVSQPTAEQAQAFKNEMQVLR",
                "SLADDK",
                "QTAQGMDYLHAKNIIHRDLK",
                "LLSDCLK",
                "ALK",
                "ALKVR",
                "FDMVQLIDVARQTAQGMDYLHAKNIIHR",
                "VR",
                "LIKGR",
                "TQHCDPEHFPFPAPANAPLQRIRSTSTPNVHMVSTTAPMDSNLIQLTGQSFSTDAAGSR",
                "GR",
                "VSQPTAEQAQAFK",
                "TR",
                "GSPSPASVSSGRK",
                "KK",
                "IGDFGLATVK",
                "DGMSVYDSLDKALK",
                "KTFFSLAFCDFCLKFLFHGFR",
                "QRTVVTVRDGMSVYDSLDK",
                "TVTAWDTAIAPLDGEELIVEVLEDVPLTMHNFVRK",
                "NLGYRDSGYYWEVPPSEVQLLKR",
                "KTR",
                "TFFSLAFCDFCLKFLFHGFR",
                "VLKVSQPTAEQAQAFK",
                "SLADDKKK",
                "GPPANGAEPSRAVGTVKVYLPNK",
                "DLKSNNIFLHEGLTVKIGDFGLATVK",
                "QRTVVTVR",
                "GRKTVTAWDTAIAPLDGEELIVEVLEDVPLTMHNFVR",
                "SLADDKK",
                "IGTGSFGTVFRGR",
                "QHEAPSNRPLNELLTPQGPSPRTQHCDPEHFPFPAPANAPLQR",
                "GYLSPDLSKISSNCPKAMR",
                "RIGTGSFGTVFRGR",
                "EERPLFPQILATIELLQRSLPK",
                "FHQHCSSKVPTVCVDMSTNRQQFYHSVQDLSGGSR",
                "FDMVQLIDVAR",
                "TFFSLAFCDFCLK",
                "HVNILLFMGFMTRPGFAIITQWCEGSSLYHHLHVADTRFDMVQLIDVARQTAQGMDYLHAK",
                "DGMSVYDSLDKALKVR",
                "ALKVRGLNQDCCVVYR",
                "SPHSK",
                "KTFFSLAFCDFCLK",
                "FHQHCSSK",
                "KSLADDK",
                "DSGYYWEVPPSEVQLLK",
                "TVVTVRDGMSVYDSLDKALK",
                "FLFHGFRCQTCGYK",
                "VRGLNQDCCVVYR",
                "IGTGSFGTVFRGRWHGDVAVK",
                "KTVTAWDTAIAPLDGEELIVEVLEDVPLTMHNFVRK",
                "TRHVNILLFMGFMTRPGFAIITQWCEGSSLYHHLHVADTR",
                "DGMSVYDSLDK",
                "TQHCDPEHFPFPAPANAPLQRIR",
                "QHEAPSNRPLNELLTPQGPSPRTQHCDPEHFPFPAPANAPLQRIR",
                "DQIIFMVGR",
                "SNNIFLHEGLTVKIGDFGLATVKTR",
                "NEMQVLRK",
                "KVK",
                "IRSTSTPNVHMVSTTAPMDSNLIQLTGQSFSTDAAGSRGGSDGTPR",
                "TRWSGAQPLEQPSGSVLWMAAEVIRMQDPNPYSFQSDVYAYGVVLYELMTGSLPYSHIGCR",
                "FHQHCSSKVPTVCVDMSTNR",
                "MEPPRGPPANGAEPSRAVGTVK",
                "WSGAQPLEQPSGSVLWMAAEVIRMQDPNPYSFQSDVYAYGVVLYELMTGSLPYSHIGCR",
                "SLPKIER",
                "MEPPR",
                "VSQPTAEQAQAFKNEMQVLRK",
                "TVVTVRDGMSVYDSLDK",
                "GLNQDCCVVYR",
                "GGSDGTPR",
                "IER",
                "ISSNCPKAMR",
                "EPPRGPPANGAEPSR",
                "TRHVNILLFMGFMTRPGFAIITQWCEGSSLYHHLHVADTRFDMVQLIDVAR",
                "GGSDGTPRGSPSPASVSSGRK",
                "HVNILLFMGFMTRPGFAIITQWCEGSSLYHHLHVADTRFDMVQLIDVAR",
                "GRWHGDVAVKVLK",
                "ISSNCPK",
                "WHGDVAVK",
                "MEPPRGPPANGAEPSR",
                "MQDPNPYSFQSDVYAYGVVLYELMTGSLPYSHIGCRDQIIFMVGR",
                "NIIHRDLK",
                "QQFYHSVQDLSGGSRQHEAPSNRPLNELLTPQGPSPR",
                "QTAQGMDYLHAK",
                "VYLPNKQRTVVTVR",
                "IGTGSFGTVFR",
                "HVNILLFMGFMTRPGFAIITQWCEGSSLYHHLHVADTR",
                "TQHCDPEHFPFPAPANAPLQR",
                "AVGTVK",
                "QR",
                "VLK",
                "MQDPNPYSFQSDVYAYGVVLYELMTGSLPYSHIGCRDQIIFMVGRGYLSPDLSK",
                "EERPLFPQILATIELLQR",
                "VSQPTAEQAQAFKNEMQVLR",
                "IERSASEPSLHR",
                "FDMVQLIDVARQTAQGMDYLHAK",
                "ISSNCPKAMRR",
                "GPPANGAEPSR",
                "DLKSNNIFLHEGLTVK",
                "AMRRLLSDCLK",
                "RIGTGSFGTVFR",
                "SNNIFLHEGLTVKIGDFGLATVK",
                "NLGYRDSGYYWEVPPSEVQLLK",
                "R",
                "IERSASEPSLHRTQADELPACLLSAAR",
                "TQADELPACLLSAAR",
                "FLFHGFR",
                "DLK",
                "SLPK",
                "STSTPNVHMVSTTAPMDSNLIQLTGQSFSTDAAGSRGGSDGTPRGSPSPASVSSGR",
                "ERKSLADDK",
                "SASEPSLHRTQADELPACLLSAARLVP",
                "LIKGRK",
                "EPPR",
                "SPHSKSPAEQR",
                "VK",
                "NIIHRDLKSNNIFLHEGLTVK",
                "NIIHR",
                "AMR",
                "ERK",
                "DQIIFMVGRGYLSPDLSKISSNCPK",
                "SPAEQRER",
                "WSGAQPLEQPSGSVLWMAAEVIRMQDPNPYSFQSDVYAYGVVLYELMTGSLPYSHIGCRDQIIFMVGR",
                "SASEPSLHR",
                "WHGDVAVKVLK",
                "TVVTVR",
                "SPAEQRERK",
                "VKNLGYRDSGYYWEVPPSEVQLLK",
                "GRWHGDVAVK",
                "GYLSPDLSKISSNCPK",
                "LLSDCLKFQREERPLFPQILATIELLQR",
                "IRSTSTPNVHMVSTTAPMDSNLIQLTGQSFSTDAAGSR",
                "WHGDVAVKVLKVSQPTAEQAQAFK",
                "TVTAWDTAIAPLDGEELIVEVLEDVPLTMHNFVRKTFFSLAFCDFCLK",
                "KSPHSKSPAEQR",
                "AVGTVKVYLPNK",
                "NLGYR",
                "LLSDCLKFQR",
                "KTVTAWDTAIAPLDGEELIVEVLEDVPLTMHNFVR",
                "EERPLFPQILATIELLQRSLPKIER",
                "GSPSPASVSSGR",
                "FLFHGFRCQTCGYKFHQHCSSK",
                "WSGAQPLEQPSGSVLWMAAEVIR",
                "VRGLNQDCCVVYRLIK",
                "AVGTVKVYLPNKQR",
                "EPPRGPPANGAEPSRAVGTVK",
                "TVTAWDTAIAPLDGEELIVEVLEDVPLTMHNFVR",
                "FQREERPLFPQILATIELLQR",
                "GLNQDCCVVYRLIKGR",
                "NEMQVLRKTR",
                "GYLSPDLSK",
                "TQADELPACLLSAARLVP",
                "AMRR",
                "SPHSKSPAEQRER",
                "ER",
                "VPTVCVDMSTNR",
                "QHEAPSNRPLNELLTPQGPSPR",
                "KKVK",
                "QQFYHSVQDLSGGSR",
                "KSLADDKK",
                "VPTVCVDMSTNRQQFYHSVQDLSGGSRQHEAPSNRPLNELLTPQGPSPR",
                "STSTPNVHMVSTTAPMDSNLIQLTGQSFSTDAAGSR",
                "MQDPNPYSFQSDVYAYGVVLYELMTGSLPYSHIGCR",
                "STSTPNVHMVSTTAPMDSNLIQLTGQSFSTDAAGSRGGSDGTPR",
                "GSPSPASVSSGRKSPHSK",
                "VYLPNK",
                "IGDFGLATVKTRWSGAQPLEQPSGSVLWMAAEVIR",
                "TRWSGAQPLEQPSGSVLWMAAEVIR",
                "VKNLGYR",
                "GRK",
                "CQTCGYKFHQHCSSKVPTVCVDMSTNR",
                "VYLPNKQR",
                "SASEPSLHRTQADELPACLLSAAR",
                "CQTCGYKFHQHCSSK",
                "IR",
                "VPTVCVDMSTNRQQFYHSVQDLSGGSR",
                "FQREERPLFPQILATIELLQRSLPK",
                "DQIIFMVGRGYLSPDLSK",
                "LIK",
                "CQTCGYK",
                "DSGYYWEVPPSEVQLLKR",
                "RLLSDCLKFQR",
                "IGDFGLATVKTR",
                "FQR",
                "LVP",
                "KSPHSK",
                "SLPKIERSASEPSLHR",
                "GPPANGAEPSRAVGTVK",
                "SPAEQR",
                "QQFYHSVQDLSGGSRQHEAPSNRPLNELLTPQGPSPRTQHCDPEHFPFPAPANAPLQR",
                "K",
                "DSGYYWEVPPSEVQLLKRIGTGSFGTVFR",
                "TFFSLAFCDFCLKFLFHGFRCQTCGYK",
                "RLLSDCLK",
                "GGSDGTPRGSPSPASVSSGR",
                "SNNIFLHEGLTVK",
                "NEMQVLR",
                "KTRHVNILLFMGFMTRPGFAIITQWCEGSSLYHHLHVADTR",
                "KVKNLGYR",
                "GLNQDCCVVYRLIK",
            },
        )
        self.assertEqual(len(digest2.peptide_to_protein_dictionary), 3710)

        protein_inference_parameters.missed_cleavages = 1
        digest3 = pyproteininference.in_silico_digest.PyteomicsDigest(
            database_path=TEST_DATABASE,
            digest_type=protein_inference_parameters.digest_type,
            missed_cleavages=protein_inference_parameters.missed_cleavages,
            reviewed_identifier_symbol=protein_inference_parameters.reviewed_identifier_symbol,
            max_peptide_length=None,
            id_splitting=True,
        )
        digest3.digest_fasta_database()

        self.assertSetEqual(
            digest3.swiss_prot_protein_set,
            {
                "ARAF_HUMAN|P10398",
                "##ARAF_HUMAN|##P10398",
                "##BRAF_HUMAN|##P15056",
                "HNRPU_HUMAN|Q00839",
                "BRAF_HUMAN|P15056",
                "##RPOC_SHIF8|##Q0SY12",
                "TCAF1_HUMAN|Q9Y4C2-2",
                "##TCAF2_HUMAN|##A6NFQ2-3",
                "RAF1_HUMAN|P04049",
                "RPOC_SHIF8|Q0SY12",
                "##TCAF2_HUMAN|##A6NFQ2",
                "TCAF2_HUMAN|A6NFQ2-3",
                "##RAF1_HUMAN|##P04049",
                "##TCAF1_HUMAN|##Q9Y4C2-2",
                "TCAF2_HUMAN|A6NFQ2",
                "##TCAF1_HUMAN|##Q9Y4C2",
                "##HNRPU_HUMAN|##Q00839",
                "##TCAF2_HUMAN|##A6NFQ2-2",
                "TCAF2_HUMAN|A6NFQ2-2",
                "TCAF1_HUMAN|Q9Y4C2",
            },
        )

        self.assertSetEqual(
            digest3.protein_to_peptide_dictionary["ARAF_HUMAN|P10398"],
            {
                "QTAQGMDYLHAKNIIHR",
                "SLADDK",
                "LLSDCLK",
                "ALK",
                "ALKVR",
                "VR",
                "LIKGR",
                "GR",
                "VSQPTAEQAQAFK",
                "TR",
                "GSPSPASVSSGRK",
                "KK",
                "IGDFGLATVK",
                "DGMSVYDSLDKALK",
                "KTR",
                "TVTAWDTAIAPLDGEELIVEVLEDVPLTMHNFVRK",
                "TFFSLAFCDFCLKFLFHGFR",
                "VLKVSQPTAEQAQAFK",
                "QRTVVTVR",
                "SLADDKK",
                "IGTGSFGTVFRGR",
                "QHEAPSNRPLNELLTPQGPSPRTQHCDPEHFPFPAPANAPLQR",
                "EERPLFPQILATIELLQRSLPK",
                "FDMVQLIDVAR",
                "TFFSLAFCDFCLK",
                "SPHSK",
                "KTFFSLAFCDFCLK",
                "FHQHCSSK",
                "KSLADDK",
                "DSGYYWEVPPSEVQLLK",
                "FLFHGFRCQTCGYK",
                "VRGLNQDCCVVYR",
                "TRHVNILLFMGFMTRPGFAIITQWCEGSSLYHHLHVADTR",
                "DGMSVYDSLDK",
                "TQHCDPEHFPFPAPANAPLQRIR",
                "DQIIFMVGR",
                "NEMQVLRK",
                "KVK",
                "FHQHCSSKVPTVCVDMSTNR",
                "WSGAQPLEQPSGSVLWMAAEVIRMQDPNPYSFQSDVYAYGVVLYELMTGSLPYSHIGCR",
                "SLPKIER",
                "MEPPR",
                "TVVTVRDGMSVYDSLDK",
                "GLNQDCCVVYR",
                "GGSDGTPR",
                "IER",
                "ISSNCPKAMR",
                "EPPRGPPANGAEPSR",
                "HVNILLFMGFMTRPGFAIITQWCEGSSLYHHLHVADTRFDMVQLIDVAR",
                "ISSNCPK",
                "WHGDVAVK",
                "MEPPRGPPANGAEPSR",
                "MQDPNPYSFQSDVYAYGVVLYELMTGSLPYSHIGCRDQIIFMVGR",
                "NIIHRDLK",
                "QQFYHSVQDLSGGSRQHEAPSNRPLNELLTPQGPSPR",
                "QTAQGMDYLHAK",
                "HVNILLFMGFMTRPGFAIITQWCEGSSLYHHLHVADTR",
                "IGTGSFGTVFR",
                "AVGTVK",
                "TQHCDPEHFPFPAPANAPLQR",
                "QR",
                "VLK",
                "IERSASEPSLHR",
                "EERPLFPQILATIELLQR",
                "VSQPTAEQAQAFKNEMQVLR",
                "FDMVQLIDVARQTAQGMDYLHAK",
                "GPPANGAEPSR",
                "DLKSNNIFLHEGLTVK",
                "RIGTGSFGTVFR",
                "SNNIFLHEGLTVKIGDFGLATVK",
                "NLGYRDSGYYWEVPPSEVQLLK",
                "R",
                "TQADELPACLLSAAR",
                "FLFHGFR",
                "DLK",
                "SLPK",
                "EPPR",
                "SPHSKSPAEQR",
                "VK",
                "NIIHR",
                "ERK",
                "SPAEQRER",
                "SASEPSLHR",
                "WHGDVAVKVLK",
                "TVVTVR",
                "GRWHGDVAVK",
                "GYLSPDLSKISSNCPK",
                "IRSTSTPNVHMVSTTAPMDSNLIQLTGQSFSTDAAGSR",
                "LLSDCLKFQR",
                "AVGTVKVYLPNK",
                "NLGYR",
                "KTVTAWDTAIAPLDGEELIVEVLEDVPLTMHNFVR",
                "GSPSPASVSSGR",
                "WSGAQPLEQPSGSVLWMAAEVIR",
                "TVTAWDTAIAPLDGEELIVEVLEDVPLTMHNFVR",
                "FQREERPLFPQILATIELLQR",
                "GYLSPDLSK",
                "TQADELPACLLSAARLVP",
                "AMRR",
                "ER",
                "VPTVCVDMSTNR",
                "QHEAPSNRPLNELLTPQGPSPR",
                "QQFYHSVQDLSGGSR",
                "STSTPNVHMVSTTAPMDSNLIQLTGQSFSTDAAGSR",
                "MQDPNPYSFQSDVYAYGVVLYELMTGSLPYSHIGCR",
                "STSTPNVHMVSTTAPMDSNLIQLTGQSFSTDAAGSRGGSDGTPR",
                "VYLPNK",
                "TRWSGAQPLEQPSGSVLWMAAEVIR",
                "VKNLGYR",
                "GRK",
                "VYLPNKQR",
                "SASEPSLHRTQADELPACLLSAAR",
                "CQTCGYKFHQHCSSK",
                "IR",
                "VPTVCVDMSTNRQQFYHSVQDLSGGSR",
                "DQIIFMVGRGYLSPDLSK",
                "LIK",
                "CQTCGYK",
                "DSGYYWEVPPSEVQLLKR",
                "FQR",
                "IGDFGLATVKTR",
                "LVP",
                "KSPHSK",
                "GPPANGAEPSRAVGTVK",
                "SPAEQR",
                "K",
                "RLLSDCLK",
                "GGSDGTPRGSPSPASVSSGR",
                "SNNIFLHEGLTVK",
                "NEMQVLR",
                "AMR",
                "GLNQDCCVVYRLIK",
            },
        )
        self.assertEqual(len(digest3.peptide_to_protein_dictionary), 2336)

        protein_inference_parameters.missed_cleavages = 0
        digest4 = pyproteininference.in_silico_digest.PyteomicsDigest(
            database_path=TEST_DATABASE,
            digest_type=protein_inference_parameters.digest_type,
            missed_cleavages=protein_inference_parameters.missed_cleavages,
            reviewed_identifier_symbol=protein_inference_parameters.reviewed_identifier_symbol,
            max_peptide_length=None,
            id_splitting=True,
        )
        digest4.digest_fasta_database()

        self.assertSetEqual(
            digest4.swiss_prot_protein_set,
            {
                "ARAF_HUMAN|P10398",
                "##ARAF_HUMAN|##P10398",
                "##BRAF_HUMAN|##P15056",
                "HNRPU_HUMAN|Q00839",
                "BRAF_HUMAN|P15056",
                "##RPOC_SHIF8|##Q0SY12",
                "TCAF1_HUMAN|Q9Y4C2-2",
                "##TCAF2_HUMAN|##A6NFQ2-3",
                "RAF1_HUMAN|P04049",
                "RPOC_SHIF8|Q0SY12",
                "##TCAF2_HUMAN|##A6NFQ2",
                "TCAF2_HUMAN|A6NFQ2-3",
                "##RAF1_HUMAN|##P04049",
                "##TCAF1_HUMAN|##Q9Y4C2-2",
                "TCAF2_HUMAN|A6NFQ2",
                "##TCAF1_HUMAN|##Q9Y4C2",
                "##HNRPU_HUMAN|##Q00839",
                "##TCAF2_HUMAN|##A6NFQ2-2",
                "TCAF2_HUMAN|A6NFQ2-2",
                "TCAF1_HUMAN|Q9Y4C2",
            },
        )

        self.assertSetEqual(
            digest4.protein_to_peptide_dictionary["ARAF_HUMAN|P10398"],
            {
                "ER",
                "SLADDK",
                "TQADELPACLLSAAR",
                "VPTVCVDMSTNR",
                "DGMSVYDSLDK",
                "FLFHGFR",
                "QHEAPSNRPLNELLTPQGPSPR",
                "DLK",
                "LLSDCLK",
                "SLPK",
                "DQIIFMVGR",
                "QQFYHSVQDLSGGSR",
                "STSTPNVHMVSTTAPMDSNLIQLTGQSFSTDAAGSR",
                "MQDPNPYSFQSDVYAYGVVLYELMTGSLPYSHIGCR",
                "EPPR",
                "VK",
                "VR",
                "NIIHR",
                "GR",
                "VSQPTAEQAQAFK",
                "VYLPNK",
                "MEPPR",
                "TR",
                "GLNQDCCVVYR",
                "IGDFGLATVK",
                "GGSDGTPR",
                "SASEPSLHR",
                "IER",
                "TVVTVR",
                "IR",
                "ISSNCPK",
                "WHGDVAVK",
                "LIK",
                "CQTCGYK",
                "NLGYR",
                "FQR",
                "FDMVQLIDVAR",
                "TFFSLAFCDFCLK",
                "GSPSPASVSSGR",
                "QTAQGMDYLHAK",
                "LVP",
                "HVNILLFMGFMTRPGFAIITQWCEGSSLYHHLHVADTR",
                "IGTGSFGTVFR",
                "AVGTVK",
                "TQHCDPEHFPFPAPANAPLQR",
                "SPAEQR",
                "WSGAQPLEQPSGSVLWMAAEVIR",
                "QR",
                "FHQHCSSK",
                "TVTAWDTAIAPLDGEELIVEVLEDVPLTMHNFVR",
                "SPHSK",
                "DSGYYWEVPPSEVQLLK",
                "K",
                "VLK",
                "EERPLFPQILATIELLQR",
                "GPPANGAEPSR",
                "SNNIFLHEGLTVK",
                "NEMQVLR",
                "GYLSPDLSK",
                "AMR",
                "ALK",
                "R",
            },
        )
        self.assertEqual(len(digest4.peptide_to_protein_dictionary), 1019)



import logging
import os
import sys

import pyproteininference
from pyproteininference.inference import Inference

logger = logging.getLogger(__name__)

# set up our logger
logging.basicConfig(
    stream=sys.stderr,
    level=logging.INFO,
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
)


class ProteinInferencePipeline(object):
    """
    This is the main Protein Inference class which houses the logic of the entire data analysis pipeline.
    Logic is executed in the [execute][pyproteininference.pipeline.ProteinInferencePipeline.execute] method.

    Attributes:
        parameter_file (str): Path to Protein Inference Yaml Parameter File.
        database_file (str): Path to Fasta database used in proteomics search.
        target_files (str/list): Path to Target Psm File (Or a list of files).
        decoy_files (str/list): Path to Decoy Psm File (Or a list of files).
        combined_files (str/list): Path to Combined Psm File (Or a list of files).
        target_directory (str): Path to Directory containing Target Psm Files.
        decoy_directory (str): Path to Directory containing Decoy Psm Files.
        combined_directory (str): Path to Directory containing Combined Psm Files.
        output_directory (str): Path to Directory where output will be written.
        output_filename (str): Path to Filename where output will be written. Will override output_directory.
        id_splitting (bool): True/False on whether to split protein IDs in the digest. Advanced usage only.
        append_alt_from_db (bool): True/False on whether to append alternative proteins from the DB digestion in
            Reader class.
        data (DataStore): [DataStore Object][pyproteininference.datastore.DataStore].
        digest (Digest): [Digest Object][pyproteininference.in_silico_digest.Digest].

    """

    def __init__(
        self,
        parameter_file,
        database_file=None,
        target_files=None,
        decoy_files=None,
        combined_files=None,
        target_directory=None,
        decoy_directory=None,
        combined_directory=None,
        output_directory=None,
        output_filename=None,
        id_splitting=False,
        append_alt_from_db=True,
    ):
        """

        Args:
            parameter_file (str): Path to Protein Inference Yaml Parameter File.
            database_file (str): Path to Fasta database used in proteomics search.
            target_files (str/list): Path to Target Psm File (Or a list of files).
            decoy_files (str/list): Path to Decoy Psm File (Or a list of files).
            combined_files (str/list): Path to Combined Psm File (Or a list of files).
            target_directory (str): Path to Directory containing Target Psm Files.
            decoy_directory (str): Path to Directory containing Decoy Psm Files.
            combined_directory (str): Path to Directory containing Combined Psm Files.
            output_filename (str): Path to Filename where output will be written. Will override output_directory.
            output_directory (str): Path to Directory where output will be written.
            id_splitting (bool): True/False on whether to split protein IDs in the digest. Advanced usage only.
            append_alt_from_db (bool): True/False on whether to append alternative proteins from the DB digestion in
                Reader class.

        Returns:
            object:

        Example:
            >>> pipeline = pyproteininference.pipeline.ProteinInferencePipeline(
            >>>     parameter_file=yaml_params,
            >>>     database_file=database,
            >>>     target_files=target,
            >>>     decoy_files=decoy,
            >>>     combined_files=combined_files,
            >>>     target_directory=target_directory,
            >>>     decoy_directory=decoy_directory,
            >>>     combined_directory=combined_directory,
            >>>     output_directory=dir_name,
            >>>     output_filename=output_filename,
            >>>     append_alt_from_db=append_alt,
            >>> )
        """

        self.parameter_file = parameter_file
        self.database_file = database_file
        self.target_files = target_files
        self.decoy_files = decoy_files
        self.combined_files = combined_files
        self.target_directory = target_directory
        self.decoy_directory = decoy_directory
        self.combined_directory = combined_directory
        self.output_directory = output_directory
        self.output_filename = output_filename
        self.id_splitting = id_splitting
        self.append_alt_from_db = append_alt_from_db
        self.data = None
        self.digest = None

        self._validate_input()

        self._set_output_directory()

        self._log_append_alt_from_db()

        self._log_id_splitting()

    def execute(self):
        """
        This method is the main driver of the data analysis for the protein inference package.
        This method calls other classes and methods that make up the protein inference pipeline.
        This includes but is not limited to:

        This method sets the data [DataStore Object][pyproteininference.datastore.DataStore] and digest
            [Digest Object][pyproteininference.in_silico_digest.Digest].

        1. Parameter file management.
        2. Digesting Fasta Database (Optional).
        3. Reading in input Psm Files.
        4. Initializing the [DataStore Object][pyproteininference.datastore.DataStore].
        5. Restricting Psms.
        6. Creating Protein objects/scoring input.
        7. Scoring Proteins.
        8. Running Protein Picker.
        9. Running Inference Methods/Grouping.
        10. Calculating Q Values.
        11. Exporting Proteins to filesystem.

        Example:
            >>> pipeline = pyproteininference.pipeline.ProteinInferencePipeline(
            >>>     parameter_file=yaml_params,
            >>>     database_file=database,
            >>>     target_files=target,
            >>>     decoy_files=decoy,
            >>>     combined_files=combined_files,
            >>>     target_directory=target_directory,
            >>>     decoy_directory=decoy_directory,
            >>>     combined_directory=combined_directory,
            >>>     output_directory=dir_name,
            >>>     output_filename=output_filename,
            >>>     append_alt_from_db=append_alt,
            >>> )
            >>> pipeline.execute()

        """
        # STEP 1: Load parameter file #
        # STEP 1: Load parameter file #
        # STEP 1: Load parameter file #
        pyproteininference_parameters = pyproteininference.parameters.ProteinInferenceParameter(
            yaml_param_filepath=self.parameter_file
        )

        # STEP 2: Start with running an In Silico Digestion #
        # STEP 2: Start with running an In Silico Digestion #
        # STEP 2: Start with running an In Silico Digestion #
        digest = pyproteininference.in_silico_digest.PyteomicsDigest(
            database_path=self.database_file,
            digest_type=pyproteininference_parameters.digest_type,
            missed_cleavages=pyproteininference_parameters.missed_cleavages,
            reviewed_identifier_symbol=pyproteininference_parameters.reviewed_identifier_symbol,
            max_peptide_length=pyproteininference_parameters.restrict_peptide_length,
            id_splitting=self.id_splitting,
        )
        if self.database_file:
            logger.info("Running In Silico Database Digest on file {}".format(self.database_file))
            digest.digest_fasta_database()
        else:
            logger.warning(
                "No Database File provided, Skipping database digest and only taking protein-peptide mapping from the "
                "input files."
            )

        # STEP 3: Read PSM Data #
        # STEP 3: Read PSM Data #
        # STEP 3: Read PSM Data #
        reader = pyproteininference.reader.GenericReader(
            target_file=self.target_files,
            decoy_file=self.decoy_files,
            combined_files=self.combined_files,
            parameter_file_object=pyproteininference_parameters,
            digest=digest,
            append_alt_from_db=self.append_alt_from_db,
        )
        reader.read_psms()

        # STEP 4: Initiate the datastore object #
        # STEP 4: Initiate the datastore object #
        # STEP 4: Initiate the datastore object #
        data = pyproteininference.datastore.DataStore(reader=reader, digest=digest)

        # Step 5: Restrict the PSM data
        # Step 5: Restrict the PSM data
        # Step 5: Restrict the PSM data
        data.restrict_psm_data()

        data.recover_mapping()
        # Step 6: Generate protein scoring input
        # Step 6: Generate protein scoring input
        # Step 6: Generate protein scoring input
        data.create_scoring_input()

        # Step 7: Remove non unique peptides if running exclusion
        # Step 7: Remove non unique peptides if running exclusion
        # Step 7: Remove non unique peptides if running exclusion
        if pyproteininference_parameters.inference_type == Inference.EXCLUSION:
            # This gets ran if we run exclusion...
            data.exclude_non_distinguishing_peptides()

        # STEP 8: Score our PSMs given a score method
        # STEP 8: Score our PSMs given a score method
        # STEP 8: Score our PSMs given a score method
        score = pyproteininference.scoring.Score(data=data)
        score.score_psms(score_method=pyproteininference_parameters.protein_score)

        # STEP 9: Run protein picker on the data
        # STEP 9: Run protein picker on the data
        # STEP 9: Run protein picker on the data
        if pyproteininference_parameters.picker:
            data.protein_picker()
        else:
            pass

        # STEP 10: Apply Inference
        # STEP 10: Apply Inference
        # STEP 10: Apply Inference
        pyproteininference.inference.Inference.run_inference(data=data, digest=digest)

        # STEP 11: Q value Calculations
        # STEP 11: Q value Calculations
        # STEP 11: Q value Calculations
        data.calculate_q_values()

        # STEP 12: Export to CSV
        # STEP 12: Export to CSV
        # STEP 12: Export to CSV
        export = pyproteininference.export.Export(data=data)
        export.export_to_csv(
            output_filename=self.output_filename,
            directory=self.output_directory,
            export_type=pyproteininference_parameters.export,
        )

        self.data = data
        self.digest = digest

        logger.info("Protein Inference Finished")

    def _validate_input(self):
        """
        Internal method that validates whether the proper input files have been defined.

        One of the following combinations must be selected as input. No more and no less:

        1. either one or multiple target_files and decoy_files.
        2. either one or multiple combined_files that include target and decoy data.
        3. a directory that contains target files (target_directory) as well as a directory that contains decoy files
            (decoy_directory).
        4. a directory that contains combined target/decoy files (combined_directory).

        Raises:
            ValueError: ValueError will occur if an improper combination of input.
        """
        if (
            self.target_files
            and self.decoy_files
            and not self.combined_files
            and not self.target_directory
            and not self.decoy_directory
            and not self.combined_directory
        ):
            logger.info("Validating input as target_files and decoy_files")
        elif (
            self.combined_files
            and not self.target_files
            and not self.decoy_files
            and not self.decoy_directory
            and not self.target_directory
            and not self.combined_directory
        ):
            logger.info("Validating input as combined_files")
        elif (
            self.target_directory
            and self.decoy_directory
            and not self.target_files
            and not self.decoy_files
            and not self.combined_directory
            and not self.combined_files
        ):
            logger.info("Validating input as target_directory and decoy_directory")
            self._transform_directory_to_files()
        elif (
            self.combined_directory
            and not self.combined_files
            and not self.decoy_files
            and not self.decoy_directory
            and not self.target_files
            and not self.target_directory
        ):
            logger.info("Validating input as combined_directory")
            self._transform_directory_to_files()
        else:
            raise ValueError(
                "To run Protein inference please supply either: "
                "(1) either one or multiple target_files and decoy_files, "
                "(2) either one or multiple combined_files that include target and decoy data"
                "(3) a directory that contains target files (target_directory) as well as a directory that "
                "contains decoy files (decoy_directory)"
                "(4) a directory that contains combined target/decoy files (combined_directory)"
            )

    def _transform_directory_to_files(self):
        """
        This internal method takes files that are in the target_directory, decoy_directory, or combined_directory and
        reassigns these files to the target_files, decoy_files, and combined_files to be used in
         [Reader][pyproteininference.reader.Reader] object.
        """
        if self.target_directory and self.decoy_directory:
            logger.info("Transforming target_directory and decoy_directory into files")
            target_files = os.listdir(self.target_directory)
            target_files_full = [
                os.path.join(self.target_directory, x) for x in target_files if x.endswith(".txt") or x.endswith(".tsv")
            ]

            decoy_files = os.listdir(self.decoy_directory)
            decoy_files_full = [
                os.path.join(self.decoy_directory, x) for x in decoy_files if x.endswith(".txt") or x.endswith(".tsv")
            ]

            self.target_files = target_files_full
            self.decoy_files = decoy_files_full

        elif self.combined_directory:
            logger.info("Transforming combined_directory into files")
            combined_files = os.listdir(self.combined_directory)
            combined_files_full = [
                os.path.join(self.combined_directory, x)
                for x in combined_files
                if x.endswith(".txt") or x.endswith(".tsv")
            ]
            self.combined_files = combined_files_full

    def _set_output_directory(self):
        """
        Internal method for setting the output directory.
        If the output_directory argument is not supplied the output directory is set as the cwd.
        """
        if not self.output_directory:
            self.output_directory = os.getcwd()
        else:
            pass

    def _log_append_alt_from_db(self):
        """
        Internal method for logging whether the user sets alternative protein append to True or False.
        """
        if self.append_alt_from_db:
            logger.info("Append Alternative Proteins from Database set to True")
        else:
            logger.info("Append Alternative Proteins from Database set to False")

    def _log_id_splitting(self):
        """
        Internal method for logging whether the user sets ID splitting to True or False.
        """
        if self.id_splitting:
            logger.info("ID Splitting for Database Digestion set to True")
        else:
            logger.info("ID Splitting for Database Digestion set to False")



import os
import tempfile
from unittest import TestCase

import yaml
from pkg_resources import resource_filename

from pyproteininference.parameters import ProteinInferenceParameter

PARAMETER_FILE = resource_filename("pyproteininference", "../tests/data/test_params_inclusion.yaml")
OUTPUT_DIR = tempfile.gettempdir()

NEW_PARAMETER_FILE = os.path.join(OUTPUT_DIR, "malformed_protein_inference_params.yaml")

SLIM_PARAMETER_FILE = resource_filename("pyproteininference", "../tests/data/test_params_slim.yaml")


class TestMalformedParameters(TestCase):
    def test_malformed_parameters(self):

        # Read params...
        # Make sure everything is as it should be...
        protein_inference_parameters = ProteinInferenceParameter(yaml_param_filepath=PARAMETER_FILE)
        self.assertIsInstance(protein_inference_parameters, ProteinInferenceParameter)

        # Read params with pyyaml
        with open(PARAMETER_FILE, "r") as stream:
            yaml_params = yaml.load(stream, Loader=yaml.Loader)

        # Edit the params to something incorrect...
        edited_yaml_params = yaml_params
        edited_yaml_params["parameters"]["digest"]["digest_type"] = "Unimplemented Digest Type"

        # Rewrite the params...
        with open(NEW_PARAMETER_FILE, 'w') as file:
            yaml.dump(edited_yaml_params, file)

        # Re-read the params with ProteinInferenceParameter...
        with self.assertRaises(ValueError):
            ProteinInferenceParameter(yaml_param_filepath=NEW_PARAMETER_FILE)

        # Read params with pyyaml
        with open(PARAMETER_FILE, "r") as stream:
            yaml_params = yaml.load(stream, Loader=yaml.Loader)

        # Edit the params to something incorrect...
        edited_yaml_params = yaml_params
        edited_yaml_params["parameters"]["general"]["export"] = None

        # Rewrite the params...
        with open(NEW_PARAMETER_FILE, 'w') as file:
            yaml.dump(edited_yaml_params, file)

        # Re-read the params with ProteinInferenceParameter...
        with self.assertRaises(ValueError):
            ProteinInferenceParameter(yaml_param_filepath=NEW_PARAMETER_FILE)

        # Read params with pyyaml
        with open(PARAMETER_FILE, "r") as stream:
            yaml_params = yaml.load(stream, Loader=yaml.Loader)

        # Edit the params to something incorrect...
        edited_yaml_params = yaml_params
        edited_yaml_params["parameters"]["general"]["fdr"] = "string"

        # Rewrite the params...
        with open(NEW_PARAMETER_FILE, 'w') as file:
            yaml.dump(edited_yaml_params, file)

        # Re-read the params with ProteinInferenceParameter...
        with self.assertRaises(ValueError):
            ProteinInferenceParameter(yaml_param_filepath=NEW_PARAMETER_FILE)

        # Read params with pyyaml
        with open(PARAMETER_FILE, "r") as stream:
            yaml_params = yaml.load(stream, Loader=yaml.Loader)

        # Edit the params to something incorrect...
        edited_yaml_params = yaml_params
        edited_yaml_params["parameters"]["digest"]["missed_cleavages"] = "not an integer"

        # Rewrite the params...
        with open(NEW_PARAMETER_FILE, 'w') as file:
            yaml.dump(edited_yaml_params, file)

        # Re-read the params with ProteinInferenceParameter...
        with self.assertRaises(ValueError):
            ProteinInferenceParameter(yaml_param_filepath=NEW_PARAMETER_FILE)

        # Read params with pyyaml
        with open(PARAMETER_FILE, "r") as stream:
            yaml_params = yaml.load(stream, Loader=yaml.Loader)

        # Edit the params to something incorrect...
        edited_yaml_params = yaml_params
        edited_yaml_params["parameters"]["general"]["picker"] = 456

        # Rewrite the params...
        with open(NEW_PARAMETER_FILE, 'w') as file:
            yaml.dump(edited_yaml_params, file)

        # Re-read the params with ProteinInferenceParameter...
        with self.assertRaises(ValueError):
            ProteinInferenceParameter(yaml_param_filepath=NEW_PARAMETER_FILE)

        # Read params with pyyaml
        with open(PARAMETER_FILE, "r") as stream:
            yaml_params = yaml.load(stream, Loader=yaml.Loader)

        # Edit the params to something incorrect...
        edited_yaml_params = yaml_params
        edited_yaml_params["parameters"]["data_restriction"]["pep_restriction"] = "string"

        # Rewrite the params...
        with open(NEW_PARAMETER_FILE, 'w') as file:
            yaml.dump(edited_yaml_params, file)

        # Re-read the params with ProteinInferenceParameter...
        with self.assertRaises(ValueError):
            ProteinInferenceParameter(yaml_param_filepath=NEW_PARAMETER_FILE)

        # Read params with pyyaml
        with open(PARAMETER_FILE, "r") as stream:
            yaml_params = yaml.load(stream, Loader=yaml.Loader)

        # Edit the params to something incorrect...
        edited_yaml_params = yaml_params
        edited_yaml_params["parameters"]["data_restriction"]["peptide_length_restriction"] = "string"

        # Rewrite the params...
        with open(NEW_PARAMETER_FILE, 'w') as file:
            yaml.dump(edited_yaml_params, file)

        # Re-read the params with ProteinInferenceParameter...
        with self.assertRaises(ValueError):
            ProteinInferenceParameter(yaml_param_filepath=NEW_PARAMETER_FILE)

        # Read params with pyyaml
        with open(PARAMETER_FILE, "r") as stream:
            yaml_params = yaml.load(stream, Loader=yaml.Loader)

        # Edit the params to something incorrect...
        edited_yaml_params = yaml_params
        edited_yaml_params["parameters"]["data_restriction"]["q_value_restriction"] = "string"

        # Rewrite the params...
        with open(NEW_PARAMETER_FILE, 'w') as file:
            yaml.dump(edited_yaml_params, file)

        # Re-read the params with ProteinInferenceParameter...
        with self.assertRaises(ValueError):
            ProteinInferenceParameter(yaml_param_filepath=NEW_PARAMETER_FILE)

        # Read params with pyyaml
        with open(PARAMETER_FILE, "r") as stream:
            yaml_params = yaml.load(stream, Loader=yaml.Loader)

        # Edit the params to something incorrect...
        edited_yaml_params = yaml_params
        edited_yaml_params["parameters"]["data_restriction"]["custom_restriction"] = "string"

        # Rewrite the params...
        with open(NEW_PARAMETER_FILE, 'w') as file:
            yaml.dump(edited_yaml_params, file)

        # Re-read the params with ProteinInferenceParameter...
        with self.assertRaises(ValueError):
            ProteinInferenceParameter(yaml_param_filepath=NEW_PARAMETER_FILE)

        # Read params with pyyaml
        with open(PARAMETER_FILE, "r") as stream:
            yaml_params = yaml.load(stream, Loader=yaml.Loader)

        # Edit the params to something incorrect...
        edited_yaml_params = yaml_params
        edited_yaml_params["parameters"]["score"]["protein_score"] = "invalid protein score"

        # Rewrite the params...
        with open(NEW_PARAMETER_FILE, 'w') as file:
            yaml.dump(edited_yaml_params, file)

        # Re-read the params with ProteinInferenceParameter...
        with self.assertRaises(ValueError):
            ProteinInferenceParameter(yaml_param_filepath=NEW_PARAMETER_FILE)

        # Read params with pyyaml
        with open(PARAMETER_FILE, "r") as stream:
            yaml_params = yaml.load(stream, Loader=yaml.Loader)

        # Edit the params to something incorrect...
        edited_yaml_params = yaml_params
        edited_yaml_params["parameters"]["score"]["psm_score_type"] = "invalid psm score type"

        # Rewrite the params...
        with open(NEW_PARAMETER_FILE, 'w') as file:
            yaml.dump(edited_yaml_params, file)

        # Re-read the params with ProteinInferenceParameter...
        with self.assertRaises(ValueError):
            ProteinInferenceParameter(yaml_param_filepath=NEW_PARAMETER_FILE)

        # Read params with pyyaml
        with open(PARAMETER_FILE, "r") as stream:
            yaml_params = yaml.load(stream, Loader=yaml.Loader)

        # Edit the params to something incorrect...
        edited_yaml_params = yaml_params
        edited_yaml_params["parameters"]["identifiers"]["decoy_symbol"] = None

        # Rewrite the params...
        with open(NEW_PARAMETER_FILE, 'w') as file:
            yaml.dump(edited_yaml_params, file)

        # Re-read the params with ProteinInferenceParameter...
        with self.assertRaises(ValueError):
            ProteinInferenceParameter(yaml_param_filepath=NEW_PARAMETER_FILE)

        # Read params with pyyaml
        with open(PARAMETER_FILE, "r") as stream:
            yaml_params = yaml.load(stream, Loader=yaml.Loader)

        # Edit the params to something incorrect...
        edited_yaml_params = yaml_params
        edited_yaml_params["parameters"]["inference"]["inference_type"] = "improper inference type"

        # Rewrite the params...
        with open(NEW_PARAMETER_FILE, 'w') as file:
            yaml.dump(edited_yaml_params, file)

        # Re-read the params with ProteinInferenceParameter...
        with self.assertRaises(ValueError):
            ProteinInferenceParameter(yaml_param_filepath=NEW_PARAMETER_FILE)

        # Read params with pyyaml
        with open(PARAMETER_FILE, "r") as stream:
            yaml_params = yaml.load(stream, Loader=yaml.Loader)

        # Edit the params to something incorrect...
        edited_yaml_params = yaml_params
        edited_yaml_params["parameters"]["inference"]["grouping_type"] = "improper grouping type"

        # Rewrite the params...
        with open(NEW_PARAMETER_FILE, 'w') as file:
            yaml.dump(edited_yaml_params, file)

        # Re-read the params with ProteinInferenceParameter...
        with self.assertRaises(ValueError):
            ProteinInferenceParameter(yaml_param_filepath=NEW_PARAMETER_FILE)

        # Read params with pyyaml
        with open(PARAMETER_FILE, "r") as stream:
            yaml_params = yaml.load(stream, Loader=yaml.Loader)

        # Edit the params to something incorrect...
        edited_yaml_params = yaml_params
        edited_yaml_params["parameters"]["peptide_centric"]["max_identifiers"] = "string"

        # Rewrite the params...
        with open(NEW_PARAMETER_FILE, 'w') as file:
            yaml.dump(edited_yaml_params, file)

        # Re-read the params with ProteinInferenceParameter...
        with self.assertRaises(ValueError):
            ProteinInferenceParameter(yaml_param_filepath=NEW_PARAMETER_FILE)

        # Read params with pyyaml
        with open(PARAMETER_FILE, "r") as stream:
            yaml_params = yaml.load(stream, Loader=yaml.Loader)

        # Edit the params to something incorrect...
        edited_yaml_params = yaml_params
        edited_yaml_params["parameters"]["parsimony"]["lp_solver"] = "improper lp solver"

        # Rewrite the params...
        with open(NEW_PARAMETER_FILE, 'w') as file:
            yaml.dump(edited_yaml_params, file)

        # Re-read the params with ProteinInferenceParameter...
        with self.assertRaises(ValueError):
            ProteinInferenceParameter(yaml_param_filepath=NEW_PARAMETER_FILE)

        # Read params with pyyaml
        with open(PARAMETER_FILE, "r") as stream:
            yaml_params = yaml.load(stream, Loader=yaml.Loader)

        # Edit the params to something incorrect...
        edited_yaml_params = yaml_params
        edited_yaml_params["parameters"]["parsimony"]["shared_peptides"] = "improper shared peptides"

        # Rewrite the params...
        with open(NEW_PARAMETER_FILE, 'w') as file:
            yaml.dump(edited_yaml_params, file)

        # Re-read the params with ProteinInferenceParameter...
        with self.assertRaises(ValueError):
            ProteinInferenceParameter(yaml_param_filepath=NEW_PARAMETER_FILE)

    def test_empty_parameters(self):
        protein_inference_parameters = ProteinInferenceParameter(yaml_param_filepath=None)

        self.assertEqual(protein_inference_parameters.digest_type, "trypsin")
        self.assertEqual(protein_inference_parameters.export, "peptides")
        self.assertEqual(protein_inference_parameters.fdr, 0.01)
        self.assertEqual(protein_inference_parameters.missed_cleavages, 3)
        self.assertEqual(protein_inference_parameters.picker, True)
        self.assertEqual(protein_inference_parameters.restrict_pep, 0.9)
        self.assertEqual(protein_inference_parameters.restrict_peptide_length, 7)
        self.assertEqual(protein_inference_parameters.restrict_q, 0.005)
        self.assertEqual(protein_inference_parameters.protein_score, "multiplicative_log")
        self.assertEqual(protein_inference_parameters.psm_score, "posterior_error_prob")
        self.assertEqual(protein_inference_parameters.psm_score_type, "multiplicative")
        self.assertEqual(protein_inference_parameters.decoy_symbol, "##")
        self.assertEqual(protein_inference_parameters.isoform_symbol, "-")
        self.assertEqual(protein_inference_parameters.reviewed_identifier_symbol, "sp|")
        self.assertEqual(protein_inference_parameters.inference_type, "peptide_centric")
        self.assertEqual(protein_inference_parameters.tag, "py_protein_inference")
        self.assertEqual(protein_inference_parameters.grouping_type, "shared_peptides")
        self.assertEqual(protein_inference_parameters.max_identifiers_peptide_centric, 5)
        self.assertEqual(protein_inference_parameters.lp_solver, "pulp")
        self.assertEqual(protein_inference_parameters.restrict_custom, None)
        self.assertEqual(protein_inference_parameters.shared_peptides, "all")

    def test_reading_slim_params(self):

        protein_inference_parameters = ProteinInferenceParameter(yaml_param_filepath=SLIM_PARAMETER_FILE)

        self.assertEqual(protein_inference_parameters.digest_type, "trypsin")
        self.assertEqual(protein_inference_parameters.export, "peptides")
        self.assertEqual(protein_inference_parameters.fdr, 0.01)
        self.assertEqual(protein_inference_parameters.missed_cleavages, 3)
        self.assertEqual(protein_inference_parameters.picker, True)
        self.assertEqual(protein_inference_parameters.restrict_pep, 0.9)
        self.assertEqual(protein_inference_parameters.restrict_peptide_length, 7)
        self.assertEqual(protein_inference_parameters.restrict_q, 0.005)
        self.assertEqual(protein_inference_parameters.protein_score, "multiplicative_log")
        self.assertEqual(protein_inference_parameters.psm_score, "posterior_error_prob")
        self.assertEqual(protein_inference_parameters.psm_score_type, "multiplicative")
        self.assertEqual(protein_inference_parameters.decoy_symbol, "decoy_")
        self.assertEqual(protein_inference_parameters.isoform_symbol, "-")
        self.assertEqual(protein_inference_parameters.reviewed_identifier_symbol, "sp|")
        self.assertEqual(protein_inference_parameters.inference_type, "inclusion")
        self.assertEqual(protein_inference_parameters.tag, "py_protein_inference")
        self.assertEqual(protein_inference_parameters.grouping_type, "shared_peptides")
        self.assertEqual(protein_inference_parameters.max_identifiers_peptide_centric, 5)
        self.assertEqual(protein_inference_parameters.lp_solver, "pulp")
        self.assertEqual(protein_inference_parameters.restrict_custom, None)
        self.assertEqual(protein_inference_parameters.shared_peptides, "all")



import logging
import re
import sys

from pyteomics import fasta, parser

logger = logging.getLogger(__name__)

# set up our logger
logging.basicConfig(
    stream=sys.stderr,
    level=logging.INFO,
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
)


class Digest(object):
    """
    The following class handles data storage of in silico digest data from a fasta formatted sequence database.

    Attributes:
        peptide_to_protein_dictionary (dict): Dictionary of peptides (keys) to protein sets (values).
        protein_to_peptide_dictionary (dict): Dictionary of proteins (keys) to peptide sets (values).
        swiss_prot_protein_set (set): Set of reviewed proteins if they are able to be distinguished from unreviewed
            proteins.
        database_path (str): Path to fasta database file to digest.
        missed_cleavages (int): The number of missed cleavages to allow.
        id_splitting (bool): True/False on whether or not to split a given regex off identifiers.
            This is used to split of "sp|" and "tr|"
            from the database protein strings as sometimes the database will contain those strings while
            the input data will have the strings split already.
            Advanced usage only.
        reviewed_identifier_symbol (str/None): Identifier that distinguishes reviewed from unreviewed proteins.
            Typically this is "sp|". Can also be None type.
        digest_type (str): can be any value in `LIST_OF_DIGEST_TYPES`.
        max_peptide_length (int): Max peptide length to keep for analysis.

    """

    TRYPSIN = "trypsin"
    LYSC = "lysc"
    LIST_OF_DIGEST_TYPES = set(parser.expasy_rules.keys())

    AA_LIST = [
        "A",
        "R",
        "N",
        "D",
        "C",
        "E",
        "Q",
        "G",
        "H",
        "I",
        "L",
        "K",
        "M",
        "F",
        "P",
        "S",
        "T",
        "W",
        "Y",
        "V",
    ]
    UNIPROT_STRS = "sp\||tr\|"  # noqa W605
    UNIPROT_STR_REGEX = re.compile(UNIPROT_STRS)
    SP_STRING = "sp|"
    METHIONINE = "M"
    ANY_AMINO_ACID = "X"

    def __init__(self):
        pass


class PyteomicsDigest(Digest):
    """
    This class represents a pyteomics implementation of an in silico digest.
    """

    def __init__(
        self,
        database_path,
        digest_type,
        missed_cleavages,
        reviewed_identifier_symbol,
        max_peptide_length,
        id_splitting=True,
    ):
        """
        The following class creates protein to peptide, peptide to protein, and reviewed protein mappings.

        The input is a fasta database, a protein inference parameter object, and whether or not to split IDs.

        This class sets important attributes for the Digest object such as: `peptide_to_protein_dictionary`,
        `protein_to_peptide_dictionary`, and `swiss_prot_protein_set`.

        Args:
            database_path (str): Path to fasta database file to digest.
            digest_type (str): Must be a value in `LIST_OF_DIGEST_TYPES`.
            missed_cleavages (int): Integer that indicates the maximum number of allowable missed cleavages from
                the ms search.
            reviewed_identifier_symbol (str/None): Symbol that indicates a reviewed identifier.
                If using Uniprot this is typically 'sp|'.
            max_peptide_length (int): The maximum length of peptides to keep for the analysis.
            id_splitting (bool): True/False on whether or not to split a given regex off identifiers.
                This is used to split of "sp|" and "tr|"
                from the database protein strings as sometimes the database will contain those
                strings while the input data will have the strings split already.
                Advanced usage only.

        Example:
            >>> digest = pyproteininference.in_silico_digest.PyteomicsDigest(
            >>>     database_path=database_file,
            >>>     digest_type='trypsin',
            >>>     missed_cleavages=2,
            >>>     reviewed_identifier_symbol='sp|',
            >>>     max_peptide_length=7,
            >>>     id_splitting=False,
            >>> )
        """
        self.peptide_to_protein_dictionary = {}
        self.protein_to_peptide_dictionary = {}
        self.swiss_prot_protein_set = set()
        self.database_path = database_path
        self.missed_cleavages = missed_cleavages
        self.id_splitting = id_splitting
        self.reviewed_identifier_symbol = reviewed_identifier_symbol
        if digest_type in self.LIST_OF_DIGEST_TYPES:
            self.digest_type = digest_type
        else:
            raise ValueError(
                "digest_type must be equal to one of the following {}".format(str(self.LIST_OF_DIGEST_TYPES))
            )
        self.max_peptide_length = max_peptide_length

    def digest_fasta_database(self):
        """
        This method reads in and prepares the fasta database for database digestion and assigns
        the several attributes for the Digest object: `peptide_to_protein_dictionary`,
        `protein_to_peptide_dictionary`, and `swiss_prot_protein_set`.

        Returns:
            None:

        Example:
            >>> digest = pyproteininference.in_silico_digest.PyteomicsDigest(
            >>>     database_path=database_file,
            >>>     digest_type='trypsin',
            >>>     missed_cleavages=2,
            >>>     reviewed_identifier_symbol='sp|',
            >>>     max_peptide_length=7,
            >>>     id_splitting=False,
            >>> )
            >>> digest.digest_fasta_database()

        """
        logger.info("Starting Pyteomics Digest...")
        pep_dict = {}
        prot_dict = {}
        sp_set = set()

        for description, sequence in fasta.read(self.database_path):
            new_peptides = parser.cleave(
                sequence,
                parser.expasy_rules[self.digest_type],
                self.missed_cleavages,
                min_length=self.max_peptide_length,
            )

            # Hopefully this splitting works...
            # IDK how robust this is...
            identifier = description.split(" ")[0]

            # Handle ID Splitting...
            if self.id_splitting:
                identifier_stripped = self.UNIPROT_STR_REGEX.sub("", identifier)
            else:
                identifier_stripped = identifier

            # If reviewed add to sp_set
            if self.reviewed_identifier_symbol:
                if identifier.startswith(self.reviewed_identifier_symbol):
                    sp_set.add(identifier_stripped)

            prot_dict[identifier_stripped] = new_peptides
            met_cleaved_peps = set()
            for peptide in new_peptides:
                pep_dict.setdefault(peptide, set()).add(identifier_stripped)
                # Need to account for potential N-term Methionine Cleavage
                if sequence.startswith(peptide) and peptide.startswith(self.METHIONINE):
                    # If our sequence starts with the current peptide... and our current peptide starts with methionine
                    # Then we remove the methionine from the peptide and add it to our dicts...
                    methionine_cleaved_peptide = peptide[1:]
                    met_cleaved_peps.add(methionine_cleaved_peptide)
            for met_peps in met_cleaved_peps:
                pep_dict.setdefault(met_peps, set()).add(identifier_stripped)
                prot_dict[identifier_stripped].add(met_peps)

        self.swiss_prot_protein_set = sp_set
        self.peptide_to_protein_dictionary = pep_dict
        self.protein_to_peptide_dictionary = prot_dict

        logger.info("Pyteomics Digest Finished...")



import os
import tempfile
from unittest import TestCase

from pkg_resources import resource_filename

import pyproteininference

TEST_DATABASE = resource_filename("pyproteininference", "../tests/data/test_database.fasta")
TARGET_FILE = resource_filename("pyproteininference", "../tests/data/test_perc_data_target.txt")
DECOY_FILE = resource_filename("pyproteininference", "../tests/data/test_perc_data_decoy.txt")
PARAMETER_FILE = resource_filename("pyproteininference", "../tests/data/test_params_heuristic.yaml")
OUTPUT_DIR = tempfile.gettempdir()
PDF_FILENAME = os.path.join(OUTPUT_DIR, "test_pyproteininference.pdf")
# OUTPUT_DIR = resource_filename('pyproteininference', '../tests/output/')
for sub_dir in ["leads", "all", "peptides", "psms", "psm_ids"]:
    if not os.path.exists(os.path.join(OUTPUT_DIR, sub_dir)):
        os.makedirs(os.path.join(OUTPUT_DIR, sub_dir))


class TestHeuristicWorkflow(TestCase):
    def test_workflow_heuristic_with_params(self):

        hp = pyproteininference.heuristic.HeuristicPipeline(
            parameter_file=PARAMETER_FILE,
            database_file=TEST_DATABASE,
            target_files=TARGET_FILE,
            decoy_files=DECOY_FILE,
            combined_files=None,
            target_directory=None,
            decoy_directory=None,
            combined_directory=None,
            output_directory=OUTPUT_DIR,
            id_splitting=True,
            pdf_filename=PDF_FILENAME,
        )

        hp.execute()

        self.assertListEqual(hp.selected_methods, ["inclusion"])

        self.assertIsInstance(hp.selected_datastores["inclusion"], pyproteininference.datastore.DataStore)

        result1 = hp.determine_optimal_inference_method(
            false_discovery_rate_threshold=0.05,
            upper_empirical_threshold=2,
            lower_empirical_threshold=1,
            pdf_filename=PDF_FILENAME,
        )

        self.assertListEqual(result1, ["parsimony"])

        result2 = hp.determine_optimal_inference_method(
            false_discovery_rate_threshold=0.1,
            upper_empirical_threshold=0.5,
            lower_empirical_threshold=0.25,
            pdf_filename=PDF_FILENAME,
        )
        self.assertListEqual(result2, ["inclusion"])

    def test_workflow_heuristic_without_params(self):

        hp = pyproteininference.heuristic.HeuristicPipeline(
            database_file=TEST_DATABASE,
            target_files=TARGET_FILE,
            decoy_files=DECOY_FILE,
            combined_files=None,
            target_directory=None,
            decoy_directory=None,
            combined_directory=None,
            output_directory=OUTPUT_DIR,
            id_splitting=True,
            pdf_filename=PDF_FILENAME,
        )

        hp.execute()

        self.assertListEqual(hp.selected_methods, ["inclusion"])

        self.assertIsInstance(hp.selected_datastores["inclusion"], pyproteininference.datastore.DataStore)

        result1 = hp.determine_optimal_inference_method(
            false_discovery_rate_threshold=0.05,
            upper_empirical_threshold=2,
            lower_empirical_threshold=1,
            pdf_filename=PDF_FILENAME,
        )

        self.assertListEqual(result1, ["parsimony"])

        result2 = hp.determine_optimal_inference_method(
            false_discovery_rate_threshold=0.1,
            upper_empirical_threshold=0.5,
            lower_empirical_threshold=0.25,
            pdf_filename=PDF_FILENAME,
        )

        self.assertListEqual(result2, ["inclusion"])

    def test_workflow_heuristic_optimal_export(self):

        hp = pyproteininference.heuristic.HeuristicPipeline(
            parameter_file=PARAMETER_FILE,
            database_file=TEST_DATABASE,
            target_files=TARGET_FILE,
            decoy_files=DECOY_FILE,
            combined_files=None,
            target_directory=None,
            decoy_directory=None,
            combined_directory=None,
            output_directory=OUTPUT_DIR,
            id_splitting=True,
            pdf_filename=PDF_FILENAME,
            output_type="optimal",
        )

        hp.execute()

        self.assertListEqual(hp.selected_methods, ["inclusion"])

        self.assertIsInstance(hp.selected_datastores["inclusion"], pyproteininference.datastore.DataStore)

        result1 = hp.determine_optimal_inference_method(
            false_discovery_rate_threshold=0.05,
            upper_empirical_threshold=2,
            lower_empirical_threshold=1,
            pdf_filename=PDF_FILENAME,
        )

        self.assertListEqual(result1, ["parsimony"])

        result3 = hp.determine_optimal_inference_method(
            false_discovery_rate_threshold=0.1,
            upper_empirical_threshold=0.5,
            lower_empirical_threshold=0.25,
            pdf_filename=PDF_FILENAME,
        )
        self.assertListEqual(result3, ["inclusion"])

    def test_workflow_heuristic_with_different_thresholds(self):

        hp = pyproteininference.heuristic.HeuristicPipeline(
            parameter_file=PARAMETER_FILE,
            database_file=TEST_DATABASE,
            target_files=TARGET_FILE,
            decoy_files=DECOY_FILE,
            combined_files=None,
            target_directory=None,
            decoy_directory=None,
            combined_directory=None,
            output_directory=OUTPUT_DIR,
            id_splitting=True,
            pdf_filename=PDF_FILENAME,
        )

        hp.execute()

        # Both inclusion/exclusion passing high upper threshold but neither peptide-centric or
        # parsimony passing lower threshold
        result1 = hp.determine_optimal_inference_method(
            false_discovery_rate_threshold=0.05,
            upper_empirical_threshold=10,
            lower_empirical_threshold=0.1,
            pdf_filename=PDF_FILENAME,
        )

        self.assertListEqual(result1, ["inclusion", "exclusion"])

        # Neither inclusion/exclusion passing low upper threshold but peptide-centric and
        # parsimony both passing high lower threshold
        result2 = hp.determine_optimal_inference_method(
            false_discovery_rate_threshold=0.1,
            upper_empirical_threshold=2,
            lower_empirical_threshold=2,
            pdf_filename=PDF_FILENAME,
        )

        self.assertListEqual(result2, ["parsimony", "peptide_centric"])

        # No methods passing thresholds
        result3 = hp.determine_optimal_inference_method(
            false_discovery_rate_threshold=0.1,
            upper_empirical_threshold=0.00001,
            lower_empirical_threshold=0.000001,
            pdf_filename=PDF_FILENAME,
        )

        self.assertListEqual(result3, ["inclusion"])



import os
import shutil
import tempfile
from unittest import TestCase

from pkg_resources import resource_filename

import pyproteininference

TEST_DATABASE = resource_filename("pyproteininference", "../tests/data/test_database.fasta")
PARAMETER_FILE = resource_filename("pyproteininference", "../tests/data/test_params_inclusion.yaml")
OUTPUT_DIR = tempfile.gettempdir()
# OUTPUT_DIR = resource_filename('pyproteininference', '../tests/output/')

TARGET_FILE = resource_filename("pyproteininference", "../tests/data/test_perc_data_target.txt")
DECOY_FILE = resource_filename("pyproteininference", "../tests/data/test_perc_data_decoy.txt")
COMBINED_FILE = resource_filename("pyproteininference", "../tests/data/combined_files/test_combined_data.txt")

temp_dir = tempfile.gettempdir()

TARGET_DIRECTORY = os.path.join(temp_dir, "target_directory")
DECOY_DIRECTORY = os.path.join(temp_dir, "decoy_directory")
if not os.path.exists(TARGET_DIRECTORY):
    os.makedirs(TARGET_DIRECTORY)
if not os.path.exists(DECOY_DIRECTORY):
    os.makedirs(DECOY_DIRECTORY)

shutil.copyfile(str(TARGET_FILE), os.path.join(TARGET_DIRECTORY, "target_file.txt"))
shutil.copyfile(DECOY_FILE, os.path.join(DECOY_DIRECTORY, "decoy_file.txt"))
COMBINED_DIRECTORY = resource_filename("pyproteininference", "../tests/data/combined_files")


class TestPipelineValidation(TestCase):
    def test_validation(self):

        # Test target and decoy file
        pipeline1 = pyproteininference.pipeline.ProteinInferencePipeline(
            parameter_file=PARAMETER_FILE,
            database_file=TEST_DATABASE,
            target_files=TARGET_FILE,
            decoy_files=DECOY_FILE,
            combined_files=None,
            target_directory=None,
            decoy_directory=None,
            combined_directory=None,
            output_directory=OUTPUT_DIR,
            id_splitting=True,
        )

        pipeline1.execute()

        self.assertEqual(len(pipeline1.data.main_data_form), 27)

        # Test combined file
        pipeline2 = pyproteininference.pipeline.ProteinInferencePipeline(
            parameter_file=PARAMETER_FILE,
            database_file=TEST_DATABASE,
            target_files=None,
            decoy_files=None,
            combined_files=COMBINED_FILE,
            target_directory=None,
            decoy_directory=None,
            combined_directory=None,
            output_directory=OUTPUT_DIR,
            id_splitting=True,
        )

        pipeline2.execute()

        self.assertEqual(len(pipeline2.data.main_data_form), 27)

        # Test target and decoy directories
        pipeline3 = pyproteininference.pipeline.ProteinInferencePipeline(
            parameter_file=PARAMETER_FILE,
            database_file=TEST_DATABASE,
            target_files=None,
            decoy_files=None,
            combined_files=None,
            target_directory=TARGET_DIRECTORY,
            decoy_directory=DECOY_DIRECTORY,
            combined_directory=None,
            output_directory=OUTPUT_DIR,
            id_splitting=True,
        )

        self.assertTrue(os.path.exists(pipeline3.target_files[0]))
        self.assertTrue(os.path.exists(pipeline3.decoy_files[0]))

        pipeline3.execute()

        self.assertEqual(len(pipeline3.data.main_data_form), 27)

        # Test combined directory
        pipeline4 = pyproteininference.pipeline.ProteinInferencePipeline(
            parameter_file=PARAMETER_FILE,
            database_file=TEST_DATABASE,
            target_files=None,
            decoy_files=None,
            combined_files=None,
            target_directory=None,
            decoy_directory=None,
            combined_directory=COMBINED_DIRECTORY,
            output_directory=OUTPUT_DIR,
            id_splitting=True,
        )

        self.assertTrue(os.path.exists(pipeline4.combined_files[0]))

        pipeline4.execute()

        self.assertEqual(len(pipeline4.data.main_data_form), 27)

        # Test Proper error reporting
        with self.assertRaises(ValueError):
            pyproteininference.pipeline.ProteinInferencePipeline(
                parameter_file=PARAMETER_FILE,
                database_file=TEST_DATABASE,
                target_files=TARGET_FILE,
                decoy_files=DECOY_FILE,
                combined_files=None,
                target_directory=TARGET_DIRECTORY,
                decoy_directory=DECOY_DIRECTORY,
                combined_directory=None,
                output_directory=OUTPUT_DIR,
                id_splitting=True,
            )

        with self.assertRaises(ValueError):
            pyproteininference.pipeline.ProteinInferencePipeline(
                parameter_file=PARAMETER_FILE,
                database_file=TEST_DATABASE,
                target_files=None,
                decoy_files=None,
                combined_files=COMBINED_FILE,
                target_directory=None,
                decoy_directory=None,
                combined_directory=COMBINED_DIRECTORY,
                output_directory=OUTPUT_DIR,
                id_splitting=True,
            )

        with self.assertRaises(ValueError):
            pyproteininference.pipeline.ProteinInferencePipeline(
                parameter_file=PARAMETER_FILE,
                database_file=TEST_DATABASE,
                target_files=TARGET_FILE,
                decoy_files=DECOY_FILE,
                combined_files=COMBINED_FILE,
                target_directory=TARGET_DIRECTORY,
                decoy_directory=DECOY_DIRECTORY,
                combined_directory=COMBINED_DIRECTORY,
                output_directory=OUTPUT_DIR,
                id_splitting=True,
            )



import logging
import sys

import yaml

from pyproteininference.export import Export
from pyproteininference.in_silico_digest import PyteomicsDigest
from pyproteininference.inference import Inference
from pyproteininference.scoring import Score

logger = logging.getLogger(__name__)

# set up our logger
logging.basicConfig(
    stream=sys.stderr,
    level=logging.INFO,
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
)


class ProteinInferenceParameter(object):
    """
    Class that handles data retrieval, storage, and validation of Protein Inference Parameters.

    Attributes:
        yaml_param_filepath (str): path to properly formatted parameter file specific to Protein Inference.
        digest_type (str): String that determines that type of digestion in silico digestion for
            [Digest object][pyproteininference.in_silico_digest.Digest]. Typically "trypsin".
        export (str): String to indicate the export type for [Export object][pyproteininference.export.Export].
            Typically this is "psms", "peptides", or "psm_ids".
        fdr (float): Float to indicate FDR filtering.
        missed_cleavages (int): Integer to determine the number of missed cleavages in the database digestion
            [Digest object][pyproteininference.in_silico_digest.Digest].
        picker (bool): True/False on whether or not to run
            the [protein picker][pyproteininference.datastore.DataStore.protein_picker] algorithm.
        restrict_pep (float/None): Float to restrict the posterior error probability values by in the PSM input.
            Used in [restrict_psm_data][pyproteininference.datastore.DataStore.restrict_psm_data].
        restrict_peptide_length (int/None): Float to restrict the peptide length values by in the PSM input.
            Used in [restrict_psm_data][pyproteininference.datastore.DataStore.restrict_psm_data].
        restrict_q (float/None): Float to restrict the q values by in the PSM input.
            Used in [restrict_psm_data][pyproteininference.datastore.DataStore.restrict_psm_data].
        restrict_custom (float/None): Float to restrict the custom values by in the PSM input.
            Used in [restrict_psm_data][pyproteininference.datastore.DataStore.restrict_psm_data].
            Filtering depends on score_type variable. If score_type is multiplicative then values that are less than
            restrict_custom are kept. If score_type is additive then values that are more than restrict_custom are kept.
        protein_score (str): String to determine the way in which Proteins are scored can be any of the SCORE_METHODS
            in [Score object][pyproteininference.scoring.Score].
        psm_score_type (str): String to determine the type of score that the PSM scores are
            (Additive or Multiplicative) can be any of the SCORE_TYPES
            in [Score object][pyproteininference.scoring.Score].
        decoy_symbol (str): String to denote decoy proteins from target proteins. IE "##".
        isoform_symbol (str): String to denote isoforms from regular proteins. IE "-". Can also be None.
        reviewed_identifier_symbol (str): String to denote a "Reviewed" Protein. Typically this is: "sp|"
            if using Uniprot Fasta database.
        inference_type (str): String to determine the inference procedure. Can be any value of INFERENCE_TYPES
            of [Inference object][pyproteininference.inference.Inference].
        tag (str): String to be added to output files.
        psm_score (str): String that indicates the PSM input score. The value should match the string in the
            input data of the score you want to use for PSM score. This score will be used in scoring methods
                here: [Score object][pyproteininference.scoring.Score].
        grouping_type (str/None): String to determine the grouping procedure. Can be any value of
            GROUPING_TYPES of [Inference object][pyproteininference.inference.Inference].
        max_identifiers_peptide_centric (int): Maximum number of identifiers to assign to a group when
            running peptide_centric inference. Typically this is 10 or 5.
        lp_solver (str/None): The LP solver to use if inference_type="Parsimony".
            Can be any value in LP_SOLVERS in the [Inference object][pyproteininference.inference.Inference].

    """

    PARENT_PARAMETER_KEY = "parameters"

    GENERAL_PARAMETER_KEY = "general"
    DATA_RESTRICTION_PARAMETER_KEY = "data_restriction"
    SCORE_PARAMETER_KEY = "score"
    IDENTIFIERS_PARAMETER_KEY = "identifiers"
    INFERENCE_PARAMETER_KEY = "inference"
    DIGEST_PARAMETER_KEY = "digest"
    PARSIMONY_PARAMETER_KEY = "parsimony"
    PEPTIDE_CENTRIC_PARAMETER_KEY = "peptide_centric"

    PARAMETER_MAIN_KEYS = {
        GENERAL_PARAMETER_KEY,
        DATA_RESTRICTION_PARAMETER_KEY,
        SCORE_PARAMETER_KEY,
        IDENTIFIERS_PARAMETER_KEY,
        INFERENCE_PARAMETER_KEY,
        DIGEST_PARAMETER_KEY,
        PARSIMONY_PARAMETER_KEY,
        PEPTIDE_CENTRIC_PARAMETER_KEY,
    }

    EXPORT_PARAMETER = "export"
    FDR_PARAMETER = "fdr"
    PICKER_PARAMETER = "picker"
    TAG_PARAMETER = "tag"

    GENERAL_PARAMETER_SUB_KEYS = {
        EXPORT_PARAMETER,
        FDR_PARAMETER,
        PICKER_PARAMETER,
        TAG_PARAMETER,
    }

    PEP_RESTRICT_PARAMETER = "pep_restriction"
    PEPTIDE_LENGTH_RESTRICT_PARAMETER = "peptide_length_restriction"
    Q_VALUE_RESTRICT_PARAMETER = "q_value_restriction"
    CUSTOM_RESTRICT_PARAMETER = "custom_restriction"

    DATA_RESTRICTION_PARAMETER_SUB_KEYS = {
        PEP_RESTRICT_PARAMETER,
        PEPTIDE_LENGTH_RESTRICT_PARAMETER,
        Q_VALUE_RESTRICT_PARAMETER,
        CUSTOM_RESTRICT_PARAMETER,
    }

    PROTEIN_SCORE_PARAMETER = "protein_score"
    PSM_SCORE_PARAMETER = "psm_score"
    PSM_SCORE_TYPE_PARAMETER = "psm_score_type"

    SCORE_PARAMETER_SUB_KEYS = {
        PROTEIN_SCORE_PARAMETER,
        PSM_SCORE_PARAMETER,
        PSM_SCORE_TYPE_PARAMETER,
    }

    DECOY_SYMBOL_PARAMETER = "decoy_symbol"
    ISOFORM_SYMBOL_PARAMETER = "isoform_symbol"
    REVIEWED_IDENTIFIER_PARAMETER = "reviewed_identifier_symbol"

    IDENTIFIER_SUB_KEYS = {
        DECOY_SYMBOL_PARAMETER,
        ISOFORM_SYMBOL_PARAMETER,
        REVIEWED_IDENTIFIER_PARAMETER,
    }

    INFERENCE_TYPE_PARAMETER = "inference_type"
    GROUPING_TYPE_PARAMETER = "grouping_type"

    INFERENCE_SUB_KEYS = {INFERENCE_TYPE_PARAMETER, GROUPING_TYPE_PARAMETER}

    DIGEST_TYPE_PARAMETER = "digest_type"
    MISSED_CLEAV_PARAMETER = "missed_cleavages"

    DIGEST_SUB_KEYS = {DIGEST_TYPE_PARAMETER, MISSED_CLEAV_PARAMETER}

    LP_SOLVER_PARAMETER = "lp_solver"
    SHARED_PEPTIDES_PARAMETER = "shared_peptides"

    PARSIMONY_SUB_KEYS = {
        LP_SOLVER_PARAMETER,
        SHARED_PEPTIDES_PARAMETER,
    }

    MAX_IDENTIFIERS_PARAMETER = "max_identifiers"

    PEPTIDE_CENTRIC_SUB_KEYS = {MAX_IDENTIFIERS_PARAMETER}

    DEFAULT_DIGEST_TYPE = "trypsin"
    DEFAULT_EXPORT = "peptides"
    DEFAULT_FDR = 0.01
    DEFAULT_MISSED_CLEAVAGES = 3
    DEFAULT_PICKER = True
    DEFAULT_RESTRICT_PEP = 0.9
    DEFAULT_RESTRICT_PEPTIDE_LENGTH = 7
    DEFAULT_RESTRICT_Q = 0.005
    DEFAULT_RESTRICT_CUSTOM = "None"
    DEFAULT_PROTEIN_SCORE = "multiplicative_log"
    DEFAULT_PSM_SCORE = "posterior_error_prob"
    DEFAULT_DECOY_SYMBOL = "##"
    DEFAULT_ISOFORM_SYMBOL = "-"
    DEFAULT_REVIEWED_IDENTIFIER_SYMBOL = "sp|"
    DEFAULT_INFERENCE_TYPE = "peptide_centric"
    DEFAULT_TAG = "py_protein_inference"
    DEFAULT_PSM_SCORE_TYPE = "multiplicative"
    DEFAULT_GROUPING_TYPE = "shared_peptides"
    DEFAULT_MAX_IDENTIFIERS_PEPTIDE_CENTRIC = 5
    DEFAULT_LP_SOLVER = "pulp"
    DEFAULT_SHARED_PEPTIDES = "all"

    def __init__(self, yaml_param_filepath, validate=True):
        """Class to store Protein Inference parameter information as an object.

        Args:
            yaml_param_filepath (str): path to properly formatted parameter file specific to Protein Inference.
            validate (bool): True/False on whether to validate the parameter file of interest.

        Returns:
            None:

        Example:
            >>> pyproteininference.parameters.ProteinInferenceParameter(
            >>>     yaml_param_filepath = "/path/to/pyproteininference_params.yaml", validate=True
            >>> )


        """
        self.yaml_param_filepath = yaml_param_filepath
        self.digest_type = self.DEFAULT_DIGEST_TYPE
        self.export = self.DEFAULT_EXPORT
        self.fdr = self.DEFAULT_FDR
        self.missed_cleavages = self.DEFAULT_MISSED_CLEAVAGES
        self.picker = self.DEFAULT_PICKER
        self.restrict_pep = self.DEFAULT_RESTRICT_PEP
        self.restrict_peptide_length = self.DEFAULT_RESTRICT_PEPTIDE_LENGTH
        self.restrict_q = self.DEFAULT_RESTRICT_Q
        self.restrict_custom = self.DEFAULT_RESTRICT_CUSTOM
        self.protein_score = self.DEFAULT_PROTEIN_SCORE
        self.psm_score_type = self.DEFAULT_PSM_SCORE_TYPE
        self.decoy_symbol = self.DEFAULT_DECOY_SYMBOL
        self.isoform_symbol = self.DEFAULT_ISOFORM_SYMBOL
        self.reviewed_identifier_symbol = self.DEFAULT_REVIEWED_IDENTIFIER_SYMBOL
        self.inference_type = self.DEFAULT_INFERENCE_TYPE
        self.tag = self.DEFAULT_TAG
        self.psm_score = self.DEFAULT_PSM_SCORE
        self.grouping_type = self.DEFAULT_GROUPING_TYPE
        self.max_identifiers_peptide_centric = self.DEFAULT_MAX_IDENTIFIERS_PEPTIDE_CENTRIC
        self.lp_solver = self.DEFAULT_LP_SOLVER
        self.shared_peptides = self.DEFAULT_SHARED_PEPTIDES
        self.validate = validate

        self.convert_to_object()

        if validate:
            self.validate_parameters()

        self._fix_none_parameters()

    def convert_to_object(self):
        """
        Function that takes a Protein Inference parameter file and converts it into a ProteinInferenceParameter object
        by assigning all Attributes of the ProteinInferenceParameter object.

        If no parameter filepath is supplied the parameter object will be loaded with default params.

        This function gets ran in the initialization of the ProteinInferenceParameter object.

        Returns:
            None:

        """
        if self.yaml_param_filepath:
            with open(self.yaml_param_filepath, "r") as stream:
                yaml_params = yaml.load(stream, Loader=yaml.Loader)

            try:
                self.digest_type = yaml_params[self.PARENT_PARAMETER_KEY][self.DIGEST_PARAMETER_KEY][
                    self.DIGEST_TYPE_PARAMETER
                ]
            except KeyError:
                logger.warning("digest_type set to default of {}".format(self.DEFAULT_DIGEST_TYPE))

            try:
                self.export = yaml_params[self.PARENT_PARAMETER_KEY][self.GENERAL_PARAMETER_KEY][self.EXPORT_PARAMETER]
            except KeyError:
                logger.warning("export set to default of {}".format(self.DEFAULT_EXPORT))

            try:
                self.fdr = yaml_params[self.PARENT_PARAMETER_KEY][self.GENERAL_PARAMETER_KEY][self.FDR_PARAMETER]
            except KeyError:
                logger.warning("fdr set to default of {}".format(self.DEFAULT_FDR))
            try:
                self.missed_cleavages = yaml_params[self.PARENT_PARAMETER_KEY][self.DIGEST_PARAMETER_KEY][
                    self.MISSED_CLEAV_PARAMETER
                ]
            except KeyError:
                logger.warning("missed_cleavages set to default of {}".format(self.DEFAULT_MISSED_CLEAVAGES))

            try:
                self.picker = yaml_params[self.PARENT_PARAMETER_KEY][self.GENERAL_PARAMETER_KEY][self.PICKER_PARAMETER]
            except KeyError:
                logger.warning("picker set to default of {}".format(self.DEFAULT_PICKER))

            try:
                self.restrict_pep = yaml_params[self.PARENT_PARAMETER_KEY][self.DATA_RESTRICTION_PARAMETER_KEY][
                    self.PEP_RESTRICT_PARAMETER
                ]
            except KeyError:
                logger.warning("restrict_pep set to default of {}".format(self.DEFAULT_RESTRICT_PEP))

            try:
                self.restrict_peptide_length = yaml_params[self.PARENT_PARAMETER_KEY][
                    self.DATA_RESTRICTION_PARAMETER_KEY
                ][self.PEPTIDE_LENGTH_RESTRICT_PARAMETER]
            except KeyError:
                logger.warning(
                    "restrict_peptide_length set to default of {}".format(self.DEFAULT_RESTRICT_PEPTIDE_LENGTH)
                )

            try:
                self.restrict_q = yaml_params[self.PARENT_PARAMETER_KEY][self.DATA_RESTRICTION_PARAMETER_KEY][
                    self.Q_VALUE_RESTRICT_PARAMETER
                ]
            except KeyError:
                logger.warning("restrict_q set to default of {}".format(self.DEFAULT_RESTRICT_Q))

            try:
                self.restrict_custom = yaml_params[self.PARENT_PARAMETER_KEY][self.DATA_RESTRICTION_PARAMETER_KEY][
                    self.CUSTOM_RESTRICT_PARAMETER
                ]
            except KeyError:
                logger.warning("restrict_custom set to default of {}".format(self.DEFAULT_RESTRICT_CUSTOM))

            try:
                self.protein_score = yaml_params[self.PARENT_PARAMETER_KEY][self.SCORE_PARAMETER_KEY][
                    self.PROTEIN_SCORE_PARAMETER
                ]
            except KeyError:
                logger.warning("protein_score set to default of {}".format(self.DEFAULT_PROTEIN_SCORE))

            try:
                self.psm_score_type = yaml_params[self.PARENT_PARAMETER_KEY][self.SCORE_PARAMETER_KEY][
                    self.PSM_SCORE_TYPE_PARAMETER
                ]
            except KeyError:
                logger.warning("psm_score_type set to default of {}".format(self.DEFAULT_PSM_SCORE_TYPE))

            try:
                self.decoy_symbol = yaml_params[self.PARENT_PARAMETER_KEY][self.IDENTIFIERS_PARAMETER_KEY][
                    self.DECOY_SYMBOL_PARAMETER
                ]
            except KeyError:
                logger.warning("decoy_symbol set to default of {}".format(self.DEFAULT_DECOY_SYMBOL))

            try:
                self.isoform_symbol = yaml_params[self.PARENT_PARAMETER_KEY][self.IDENTIFIERS_PARAMETER_KEY][
                    self.ISOFORM_SYMBOL_PARAMETER
                ]
            except KeyError:
                logger.warning("isoform_symbol set to default of {}".format(self.DEFAULT_ISOFORM_SYMBOL))

            try:
                self.reviewed_identifier_symbol = yaml_params[self.PARENT_PARAMETER_KEY][
                    self.IDENTIFIERS_PARAMETER_KEY
                ][self.REVIEWED_IDENTIFIER_PARAMETER]
            except KeyError:
                logger.warning(
                    "reviewed_identifier_symbol set to default of {}".format(self.DEFAULT_REVIEWED_IDENTIFIER_SYMBOL)
                )

            try:
                self.inference_type = yaml_params[self.PARENT_PARAMETER_KEY][self.INFERENCE_PARAMETER_KEY][
                    self.INFERENCE_TYPE_PARAMETER
                ]
            except KeyError:
                logger.warning("inference_type set to default of {}".format(self.DEFAULT_INFERENCE_TYPE))

            try:
                self.tag = yaml_params[self.PARENT_PARAMETER_KEY][self.GENERAL_PARAMETER_KEY][self.TAG_PARAMETER]
            except KeyError:
                logger.warning("tag set to default of {}".format(self.DEFAULT_TAG))

            try:
                self.psm_score = yaml_params[self.PARENT_PARAMETER_KEY][self.SCORE_PARAMETER_KEY][
                    self.PSM_SCORE_PARAMETER
                ]
            except KeyError:
                logger.warning("psm_score set to default of {}".format(self.DEFAULT_PSM_SCORE))

            try:
                self.grouping_type = yaml_params[self.PARENT_PARAMETER_KEY][self.INFERENCE_PARAMETER_KEY][
                    self.GROUPING_TYPE_PARAMETER
                ]
            except KeyError:
                logger.warning("grouping_type set to default of {}".format(self.DEFAULT_GROUPING_TYPE))

            try:
                self.max_identifiers_peptide_centric = yaml_params[self.PARENT_PARAMETER_KEY][
                    self.PEPTIDE_CENTRIC_PARAMETER_KEY
                ][self.MAX_IDENTIFIERS_PARAMETER]
            except KeyError:
                logger.warning(
                    "max_identifiers_peptide_centric set to default of {}".format(
                        self.DEFAULT_MAX_IDENTIFIERS_PEPTIDE_CENTRIC
                    )
                )

            try:
                self.lp_solver = yaml_params[self.PARENT_PARAMETER_KEY][self.PARSIMONY_PARAMETER_KEY][
                    self.LP_SOLVER_PARAMETER
                ]
            except KeyError:
                logger.warning("lp_solver set to default of {}".format(self.DEFAULT_LP_SOLVER))
            try:
                # Do try except here to make old param files backwards compatible
                self.shared_peptides = yaml_params[self.PARENT_PARAMETER_KEY][self.PARSIMONY_PARAMETER_KEY][
                    self.SHARED_PEPTIDES_PARAMETER
                ]
            except KeyError:
                logger.warning("shared_peptides set to default of {}".format(self.DEFAULT_SHARED_PEPTIDES))

        else:
            logger.warning("Yaml parameter file not found, all parameters set to default")

    def validate_parameters(self):
        """
        Class method to validate all parameters.

        Returns:
            None:

        """
        # Run all of the parameter validations
        self._validate_digest_type()
        self._validate_export_type()
        self._validate_floats()
        self._validate_bools()
        self._validate_score_type()
        self._validate_score_method()
        self._validate_score_combination()
        self._validate_inference_type()
        self._validate_grouping_type()
        self._validate_max_id()
        self._validate_lp_solver()
        self._validate_identifiers()
        self._validate_parsimony_shared_peptides()

    def _validate_digest_type(self):
        """
        Internal ProteinInferenceParameter method to validate the digest type.
        """
        # Make sure we have a valid digest type
        if self.digest_type in PyteomicsDigest.LIST_OF_DIGEST_TYPES:
            logger.info("Using digest type '{}'".format(self.digest_type))
        else:
            raise ValueError(
                "Digest Type '{}' not supported, please use one of the following enyzme digestions: '{}'".format(
                    self.digest_type, ", ".join(PyteomicsDigest.LIST_OF_DIGEST_TYPES)
                )
            )

    def _validate_export_type(self):
        """
        Internal ProteinInferenceParameter method to validate the export type.
        """
        # Make sure we have a valid export type
        if self.export in Export.EXPORT_TYPES:
            logger.info("Using Export type '{}'".format(self.export))
        else:
            raise ValueError(
                "Export Type '{}' not supported, please use one of the following export types: '{}'".format(
                    self.export, ", ".join(Export.EXPORT_TYPES)
                )
            )
        pass

    def _validate_floats(self):
        """
        Internal ProteinInferenceParameter method to validate floats.
        """
        # Validate that FDR, cleavages, and restrict values are all floats and or ints if they need to be

        try:
            if 0 <= float(self.fdr) <= 1:
                logger.info("FDR Input {}".format(self.fdr))

        except ValueError:
            raise ValueError("FDR must be a decimal between 0 and 1, FDR provided: {}".format(self.fdr))

        try:
            if 0 <= float(self.restrict_pep) <= 1:
                logger.info("PEP restriction {}".format(self.restrict_pep))

        except ValueError:
            if not self.restrict_pep or self.restrict_pep.lower() == "none":
                self.restrict_pep = None
                logger.info("Not restrict by PEP Value")
            else:
                raise ValueError(
                    "PEP restriction must be a decimal between 0 and 1, PEP restriction provided: {}".format(
                        self.restrict_pep
                    )
                )

        try:
            if 0 <= float(self.restrict_q) <= 1:
                logger.info("Q Value restriction {}".format(self.restrict_q))

        except ValueError:
            if not self.restrict_q or self.restrict_q.lower() == "none":
                self.restrict_q = None
                logger.info("Not restrict by Q Value")
            else:
                raise ValueError(
                    "Q Value restriction must be a decimal between 0 and 1, Q Value restriction provided: {}".format(
                        self.restrict_q
                    )
                )

        try:
            int(self.missed_cleavages)
            logger.info("Missed Cleavages selected: {}".format(self.missed_cleavages))
        except ValueError:
            raise ValueError(
                "Missed Cleavages must be an integer, Provided Missed Cleavages value: {}".format(self.missed_cleavages)
            )

        try:
            int(self.restrict_peptide_length)
            logger.info("Peptide Length Restriction: Len {}".format(self.restrict_peptide_length))
        except ValueError:
            if not self.restrict_peptide_length or self.restrict_peptide_length.lower() == "none":
                self.restrict_peptide_length = None
                logger.info("Not Restricting by Peptide Length")
            else:
                raise ValueError(
                    "Peptide Length Restriction must be an integer, "
                    "Provided Peptide Length Restriction value: {}".format(self.restrict_peptide_length)
                )

        try:
            float(self.restrict_custom)
            logger.info("Custom restriction {}".format(self.restrict_custom))
        except ValueError or TypeError:
            if not self.restrict_custom or self.restrict_custom.lower() == "none":
                self.restrict_custom = None
                logger.info("Not Restricting by Custom Value")
            else:
                raise ValueError(
                    "Custom restriction must be a number, Custom restriction provided: {}".format(self.restrict_custom)
                )

    def _validate_bools(self):
        """
        Internal ProteinInferenceParameter method to validate the bools.
        """
        # Make sure picker is a bool
        if type(self.picker) == bool:
            if self.picker:
                logger.info("Parameters loaded to run Picker")
            else:
                logger.info("Parameters loaded to NOT run Picker")
        else:
            raise ValueError(
                "Picker Variable must be set to True or False, Picker Variable provided: {}".format(self.picker)
            )

    def _validate_score_method(self):
        """
        Internal ProteinInferenceParameter method to validate the score method.
        """
        # Make sure we have the score method defined in code to use...
        if self.protein_score in Score.SCORE_METHODS:
            logger.info("Using Score Method '{}'".format(self.protein_score))
        else:
            raise ValueError(
                "Score Method '{}' not supported, "
                "please use one of the following Score Methods: '{}'".format(
                    self.protein_score, ", ".join(Score.SCORE_METHODS)
                )
            )

    def _validate_score_type(self):
        """
        Internal ProteinInferenceParameter method to validate the score type.
        """
        # Make sure score type is multiplicative or additive
        if self.psm_score_type in Score.SCORE_TYPES:
            logger.info("Using Score Type '{}'".format(self.psm_score_type))
        else:
            raise ValueError(
                "Score Type '{}' not supported, "
                "please use one of the following Score Types: '{}'".format(
                    self.psm_score_type, ", ".join(Score.SCORE_TYPES)
                )
            )

    def _validate_score_combination(self):
        """
        Internal ProteinInferenceParameter method to validate combination of score method and score type.
        """
        # Check to see if combination of score (column), method(multiplicative log, additive),
        # and score type (multiplicative/additive) is possible...
        # This will be super custom

        if self.psm_score_type == Score.ADDITIVE_SCORE_TYPE and self.protein_score != Score.ADDITIVE:
            raise ValueError(
                "If Score type is 'additive' (Higher PSM score is better) then you must use the 'additive' score method"
            )

        elif self.psm_score_type == Score.MULTIPLICATIVE_SCORE_TYPE and self.protein_score == Score.ADDITIVE:
            raise ValueError(
                "If Score type is 'multiplicative' (Lower PSM score is better) "
                "then you must NOT use the 'additive' score method please "
                "select one of the following score methods: {}".format(
                    ", ".join([x for x in Score.SCORE_METHODS if x != "additive"])
                )
            )

        else:
            logger.info(
                "Combination of Score Type: '{}' and Score Method: '{}' is Ok".format(
                    self.psm_score_type, self.protein_score
                )
            )

    def _validate_inference_type(self):
        """
        Internal ProteinInferenceParameter method to validate the inference type.
        """
        # Check if its parsimony, exclusion, inclusion, none
        if self.inference_type in Inference.INFERENCE_TYPES:
            logger.info("Using inference type '{}'".format(self.inference_type))
        else:
            raise ValueError(
                "Inferece Type '{}' not supported, please use one of the following Inferece Types: '{}'".format(
                    self.inference_type, ", ".join(Inference.INFERENCE_TYPES)
                )
            )

    def _validate_grouping_type(self):
        """
        Internal ProteinInferenceParameter method to validate the grouping type.
        """
        # Check if its parsimony, exclusion, inclusion, none
        if self.grouping_type in Inference.GROUPING_TYPES:
            logger.info("Using Grouping type '{}'".format(self.grouping_type))
        else:
            if self.grouping_type.lower() == "none" or not self.grouping_type:
                self.grouping_type = None
                logger.info("Using Grouping type: None")
            else:

                raise ValueError(
                    "Grouping Type '{}' not supported, please use one of the following Grouping Types: '{}'".format(
                        self.grouping_type, Inference.GROUPING_TYPES
                    )
                )

    def _validate_max_id(self):
        """
        Internal ProteinInferenceParameter method to validate the max peptide centric id.
        """
        # Check if max_identifiers_peptide_centric param is an INT
        if type(self.max_identifiers_peptide_centric) == int:
            logger.info(
                "Max Number of Indentifiers for Peptide Centric Inference: '{}'".format(
                    self.max_identifiers_peptide_centric
                )
            )
        else:
            raise ValueError(
                "Max Number of Indentifiers for Peptide Centric Inference must be an integer, "
                "provided value: {}".format(self.max_identifiers_peptide_centric)
            )

    def _validate_lp_solver(self):
        """
        Internal ProteinInferenceParameter method to validate the lp solver.
        """
        # Check if its pulp or None
        if self.lp_solver in Inference.LP_SOLVERS:
            logger.info("Using LP Solver '{}'".format(self.lp_solver))
        else:
            if self.lp_solver.lower() == "none" or not self.lp_solver:
                self.lp_solver = None
                logger.info("Setting LP Solver to None")
            else:
                raise ValueError(
                    "LP Solver '{}' not supported, please use one of the following LP Solvers: '{}'".format(
                        self.lp_solver, ", ".join(Inference.LP_SOLVERS)
                    )
                )

    def _validate_parsimony_shared_peptides(self):
        """
        Internal ProteinInferenceParameter method to validate the shared peptides parameter.
        """
        # Check if its all, best, or none
        if self.shared_peptides in Inference.SHARED_PEPTIDE_TYPES:
            logger.info("Using Shared Peptide types '{}'".format(self.shared_peptides))
        else:
            if self.shared_peptides.lower() == "none" or not self.shared_peptides:
                self.shared_peptides = None
                logger.info("Setting Shared Peptide type to None")
            else:
                raise ValueError(
                    "Shared Peptide types '{}' not supported, please use one of the following "
                    "Shared Peptide types: '{}'".format(self.shared_peptides, Inference.SHARED_PEPTIDE_TYPES)
                )

    def _validate_identifiers(self):
        """
        Internal ProteinInferenceParameter method to validate the decoy symbol, isoform symbol,
        and reviewed identifier symbol.

        """
        if type(self.decoy_symbol) == str:
            logger.info("Decoy Symbol set to: '{}'".format(self.decoy_symbol))
        else:
            raise ValueError("Decoy Symbol must be a string, provided value: {}".format(self.decoy_symbol))

        if type(self.isoform_symbol) == str:
            logger.info("Isoform Symbol set to: '{}'".format(self.isoform_symbol))
            if self.isoform_symbol.lower() == "none" or not self.isoform_symbol:
                self.isoform_symbol = None
                logger.info("Isoform Symbol set to None")
        else:
            if self.isoform_symbol:
                self.isoform_symbol = None
                logger.info("Isoform Symbol set to None")
            raise ValueError("Isoform Symbol must be a string, provided value: {}".format(self.isoform_symbol))

        if type(self.reviewed_identifier_symbol) == str:
            logger.info("Reviewed Identifier Symbol set to: '{}'".format(self.reviewed_identifier_symbol))
            if self.reviewed_identifier_symbol.lower() == "none" or not self.reviewed_identifier_symbol:
                self.reviewed_identifier_symbol = None
                logger.info("Reviewed Identifier Symbol set to None")
        else:
            if not self.reviewed_identifier_symbol:
                self.reviewed_identifier_symbol = None
                logger.info("Reviewed Identifier Symbol set to None")
            raise ValueError(
                "Reviewed Identifier Symbol must be a string, provided value: {}".format(
                    self.reviewed_identifier_symbol
                )
            )

    def _validate_parameter_shape(self, yaml_params):
        """
        Internal ProteinInferenceParameter method to validate shape of the parameter file by checking to make sure
         that all necessary main parameter fields are defined.
        """
        if self.PARENT_PARAMETER_KEY in yaml_params.keys():
            logger.info("Main Parameter Key is Present")
        else:
            raise ValueError(
                "Key {} needs to be defined as the outermost parameter group".format(self.PARENT_PARAMETER_KEY)
            )

        if self.PARAMETER_MAIN_KEYS.issubset(yaml_params[self.PARENT_PARAMETER_KEY]):
            logger.info("All Sub Parameter Keys Present")
        else:
            raise ValueError(
                "All of the following values: {}. Need to be Sub Parameters in the Yaml Parameter file".format(
                    ", ".join(self.PARAMETER_MAIN_KEYS),
                )
            )

        try:
            general_params = yaml_params[self.PARENT_PARAMETER_KEY][self.GENERAL_PARAMETER_KEY]
            for gkey in self.GENERAL_PARAMETER_SUB_KEYS:
                if gkey in general_params.keys():
                    pass
                else:
                    raise ValueError(
                        "General Sub Parameter '{}' is not found in the parameter file. "
                        "Please add it as a sub parameter of the general parameter field".format(gkey)
                    )

        except KeyError:
            raise ValueError("'general' sub Parameter not defined in the parameter file")

        try:
            data_res_params = yaml_params[self.PARENT_PARAMETER_KEY][self.DATA_RESTRICTION_PARAMETER_KEY]
            for drkey in self.DATA_RESTRICTION_PARAMETER_SUB_KEYS:
                if drkey in data_res_params.keys():
                    pass
                else:
                    raise ValueError(
                        "Data Restriction Sub Parameter '{}' is not found in the parameter file. "
                        "Please add it as a sub parameter of the data_restriction parameter field".format(drkey)
                    )

        except KeyError:
            raise ValueError("'data_restriction' sub Parameter not defined in the parameter file")

        try:
            score_params = yaml_params[self.PARENT_PARAMETER_KEY][self.SCORE_PARAMETER_KEY]
            for skey in self.SCORE_PARAMETER_SUB_KEYS:
                if skey in score_params.keys():
                    pass
                else:
                    raise ValueError(
                        "Score Sub Parameter '{}' is not found in the parameter file. "
                        "Please add it as a sub parameter of the score parameter field".format(skey)
                    )

        except KeyError:
            raise ValueError("'score' sub Parameter not defined in the parameter file")

        try:
            id_params = yaml_params[self.PARENT_PARAMETER_KEY][self.IDENTIFIERS_PARAMETER_KEY]
            for ikey in self.IDENTIFIER_SUB_KEYS:
                if ikey in id_params.keys():
                    pass
                else:
                    raise ValueError(
                        "Identifiers Sub Parameter '{}' is not found in the parameter file. "
                        "Please add it as a sub parameter of the identifiers parameter field".format(ikey)
                    )

        except KeyError:
            raise ValueError("'identifiers' sub Parameter not defined in the parameter file")

        try:
            inf_params = yaml_params[self.PARENT_PARAMETER_KEY][self.INFERENCE_PARAMETER_KEY]
            for infkey in self.INFERENCE_SUB_KEYS:
                if infkey in inf_params.keys():
                    pass
                else:
                    raise ValueError(
                        "Inference Sub Parameter '{}' is not found in the parameter file. "
                        "Please add it as a sub parameter of the inference parameter field".format(infkey)
                    )

        except KeyError:
            raise ValueError("'inference' sub Parameter not defined in the parameter file")

        try:
            digest_params = yaml_params[self.PARENT_PARAMETER_KEY][self.DIGEST_PARAMETER_KEY]
            for dkey in self.DIGEST_SUB_KEYS:
                if dkey in digest_params.keys():
                    pass
                else:
                    raise ValueError(
                        "Digest Sub Parameter '{}' is not found in the parameter file. "
                        "Please add it as a sub parameter of the digest parameter field".format(dkey)
                    )

        except KeyError:
            raise ValueError("'digest' sub Parameter not defined in the parameter file")

        try:
            parsimony_params = yaml_params[self.PARENT_PARAMETER_KEY][self.PARSIMONY_PARAMETER_KEY]
            for pkey in self.PARSIMONY_SUB_KEYS:
                if pkey in parsimony_params.keys():
                    pass
                else:
                    raise ValueError(
                        "Parsimony Sub Parameter '{}' is not found in the parameter file. "
                        "Please add it as a sub parameter of the parsimony parameter field".format(pkey)
                    )

        except KeyError:
            raise ValueError("'parsimony' sub Parameter not defined in the parameter file")

        try:
            pep_cen_params = yaml_params[self.PARENT_PARAMETER_KEY][self.PEPTIDE_CENTRIC_PARAMETER_KEY]
            for pckey in self.PEPTIDE_CENTRIC_SUB_KEYS:
                if pckey in pep_cen_params.keys():
                    pass
                else:
                    raise ValueError(
                        "Peptide Centric Sub Parameter '{}' is not found in the parameter file. "
                        "Please add it as a sub parameter of the peptide_centric parameter field".format(pckey)
                    )

        except KeyError:
            raise ValueError("'peptide_centric' sub Parameter not defined in the parameter file")

    def override_q_restrict(self, data):
        """
        ProteinInferenceParameter method to override restrict_q if the input data does not contain q values.

        Args:
            data (DataStore): [DataStore Object][pyproteininference.datastore.DataStore].

        """
        data_has_q = data.input_has_q()
        if data_has_q:
            pass
        else:
            if self.restrict_q:
                logger.warning("No Q values found in the input data, overriding parameters to not filter on Q value")
                self.restrict_q = None

    def override_pep_restrict(self, data):
        """
        ProteinInferenceParameter method to override restrict_pep if the input data does not contain pep values.

        Args:
            data (DataStore): [DataStore Object][pyproteininference.datastore.DataStore].

        """
        data_has_pep = data.input_has_pep()
        if data_has_pep:
            pass
        else:
            if self.restrict_pep:
                logger.warning(
                    "No Pep values found in the input data, overriding parameters to not filter on Pep value"
                )
                self.restrict_pep = None

    def override_custom_restrict(self, data):
        """
        ProteinInferenceParameter method to override restrict_custom if
        the input data does not contain custom score values.

        Args:
            data (DataStore): [DataStore Object][pyproteininference.datastore.DataStore].

        """
        data_has_custom = data.input_has_custom()
        if data_has_custom:
            pass
        else:
            if self.restrict_custom:
                logger.warning(
                    "No Custom values found in the input data, overriding parameters to not filter on Custom value"
                )
                self.restrict_custom = None

    def fix_parameters_from_datastore(self, data):
        """
        ProteinInferenceParameter method to override restriction values in the
        parameter file if those scores do not exist in the input files.

        Args:
            data (DataStore): [DataStore Object][pyproteininference.datastore.DataStore].

        """

        self.override_q_restrict(data=data)
        self.override_pep_restrict(data=data)
        self.override_custom_restrict(data=data)

    def _fix_none_parameters(self):
        """
        Internal ProteinInferenceParameter method to fix parameters that have been defined as None.
        These get read in as strings with YAML reader and need to be converted to None type.
        """

        self._fix_grouping_type()
        self._fix_lp_solver()
        self._fix_shared_peptides()

    def _fix_grouping_type(self):
        """
        Internal ProteinInferenceParameter method to override grouping type for None value.
        """
        if self.grouping_type in ["None", "none", None]:
            self.grouping_type = None

    def _fix_lp_solver(self):
        """
        Internal ProteinInferenceParameter method to override lp_solver for None value.
        """
        if self.lp_solver in ["None", "none", None]:
            self.lp_solver = None

    def _fix_shared_peptides(self):
        """
        Internal ProteinInferenceParameter method to override shared_peptides for None value.
        """
        if self.shared_peptides in ["None", "none", None]:
            self.shared_peptides = None
